#include "DeviceConfigureFrame.h"
#include "DeviceHandlerBlueDevice.h"
#include "DeviceHandlerBlueFOX.h"
#include "DeviceHandlerHYPERION.h"
#include "DeviceListCtrl.h"
#include <limits>
#include <memory>
#include <apps/Common/Info.h>
#include "LogOutputHandlerDlg.h"
#include <apps/Common/mvIcon.xpm>
#include <string>
#include "wx/config.h"
#include "wx/image.h"
#include "wx/numdlg.h"
#include <apps/Common/wxAbstraction.h>

using namespace mvIMPACT::acquire;
using namespace std;

//-----------------------------------------------------------------------------
class MyApp : public wxApp
//-----------------------------------------------------------------------------
{
public:
	virtual bool OnInit();
};

// ----------------------------------------------------------------------------
// event tables and other macros for wxWidgets
// ----------------------------------------------------------------------------

// the event tables connect the wxWidgets events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(DeviceConfigureFrame, wxFrame)
	EVT_MENU(miAbout, DeviceConfigureFrame::OnAbout)
	EVT_MENU(miQuit, DeviceConfigureFrame::OnQuit)
	EVT_MENU(miAction_SetID, DeviceConfigureFrame::OnSetID)
	EVT_MENU(miAction_UpdateFW, DeviceConfigureFrame::OnUpdateFirmware)
	EVT_MENU(miAction_UpdateKernelDriver, DeviceConfigureFrame::OnUpdateKernelDriver)
	EVT_MENU(miAction_ConfigureLogOutput, DeviceConfigureFrame::OnConfigureLogOutput)
	EVT_MENU(miAction_UpdateDeviceList, DeviceConfigureFrame::OnUpdateDeviceList)
	EVT_TIMER(wxID_ANY, DeviceConfigureFrame::OnTimer)
END_EVENT_TABLE()

// Create a new application object: this macro will allow wxWidgets to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also declares the accessor function
// wxGetApp() which will return the reference of the right type (i.e. MyApp and
// not wxApp)
IMPLEMENT_APP(MyApp)

//-----------------------------------------------------------------------------
// `Main program' equivalent: the program execution "starts" here
bool MyApp::OnInit()
//-----------------------------------------------------------------------------
{
	// Create the main application window
	DeviceConfigureFrame *frame = new DeviceConfigureFrame( wxString::Format( wxT("Configuration tool for MATRIX VISION GmbH devices(%s)"), VERSION_STRING ), wxDefaultPosition, wxDefaultSize );
	frame->Show(true);
	SetTopWindow(frame);
	// success: wxApp::OnRun() will be called which will enter the main message
	// loop and the application will run. If we returned false here, the
	// application would exit immediately.
	return true;
}

//-----------------------------------------------------------------------------
class FramePositionStorage
//-----------------------------------------------------------------------------
{
	wxConfigBase* m_pConfig;
	wxWindow* m_pWin;
public:
	FramePositionStorage( wxWindow* pWin ) : m_pConfig(wxConfigBase::Get()), m_pWin(pWin) {}
	void store( void ) const
	{
		int Height, Width, XPos, YPos;
		m_pWin->GetSize( &Width, &Height );
		m_pWin->GetPosition( &XPos, &YPos );
		m_pConfig->Write( wxT("/MainFrame/h"), Height );
		m_pConfig->Write( wxT("/MainFrame/w"), Width );
		m_pConfig->Write( wxT("/MainFrame/x"), XPos );
		m_pConfig->Write( wxT("/MainFrame/y"), YPos );
	}
	wxRect restore( const wxRect& defaultDimensions ) const
	{
		wxRect rect;
		rect.height = m_pConfig->Read( wxT("/MainFrame/h"), defaultDimensions.height );
		rect.width = m_pConfig->Read( wxT("/MainFrame/w"), defaultDimensions.width );
		rect.x = m_pConfig->Read( wxT("/MainFrame/x"), defaultDimensions.x );
		rect.y = m_pConfig->Read( wxT("/MainFrame/y"), defaultDimensions.y );
		int displayWidth = 0;
		int displayHeight = 0;
		wxDisplaySize( &displayWidth, &displayHeight );
		if( ( rect.x >= displayWidth ) || ( rect.x < 0 ) )
		{
			rect.x = 0;
		}
		if( ( rect.y >= displayHeight ) || ( rect.y < 0 ) )
		{
			rect.y = 0;
		}
		return rect;
	}
};

//=============================================================================
//================= Implementation DeviceConfigureFrame =======================
//=============================================================================
//-----------------------------------------------------------------------------
DeviceConfigureFrame::DeviceConfigureFrame( const wxString& title, const wxPoint& pos, const wxSize& size )
	: wxFrame((wxFrame *)NULL, wxID_ANY, title, pos, size),
	m_pDevListCtrl(0), m_pLogWindow(0), m_lastDevMgrChangedCount(numeric_limits<unsigned int>::max())
//-----------------------------------------------------------------------------
{
	m_deviceHandlerFactory.Register( wxString(wxT("mvBlueCOUGAR")), DeviceHandlerBlueDevice::Create);
	m_deviceHandlerFactory.Register( wxString(wxT("mvBlueLYNX")), DeviceHandlerBlueDevice::Create);
	m_deviceHandlerFactory.Register( wxString(wxT("mvBlueFOX")), DeviceHandlerBlueFOX::Create);
	m_deviceHandlerFactory.Register( wxString(wxT("mvHYPERION")), DeviceHandlerHYPERION::Create);

	wxMenu *menuAction = new wxMenu;
	m_pMIActionSetID = menuAction->Append( miAction_SetID, wxT("&Set ID\tCTRL+S") );
	m_pMIActionUpdateFW = menuAction->Append( miAction_UpdateFW, wxT("Update &firmware\tCTRL+F") );
	m_pMIActionUpdateKernelDriver = menuAction->Append( miAction_UpdateKernelDriver, wxT("Update &kernel driver\tCTRL+K") );

	menuAction->AppendSeparator();
	menuAction->Append( miAction_ConfigureLogOutput, wxT("&Configure log output\tCTRL+C") );
	m_pMIActionUpdateDeviceList = menuAction->Append( miAction_UpdateDeviceList, wxT("&Update device list\tALT+CTRL+U") );
	menuAction->AppendSeparator();

	menuAction->Append( miQuit, wxT("E&xit\tALT+X"));

	wxMenu *menuHelp = new wxMenu;
	menuHelp->Append( miAbout, wxT("&About mvDeviceConfigure") );

	// create a menu bar...
	wxMenuBar *menuBar = new wxMenuBar;
	// ... add all the menu items...
	menuBar->Append( menuAction, wxT("&Action") );
	menuBar->Append( menuHelp, wxT("&Help") );
	// ... and attach this menu bar to the frame
	SetMenuBar(menuBar);

	// define the applications icon
	wxIcon icon(mvIcon_xpm);
	SetIcon( icon );

	wxPanel* pPanel = new wxPanel(this, wxID_ANY);

	m_pDevListCtrl = new DeviceListCtrl(pPanel, LIST_CTRL, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL | wxSUNKEN_BORDER, this);
	m_pDevListCtrl->InsertColumn( lcFamily, wxT("family") );
	m_pDevListCtrl->InsertColumn( lcProduct, wxT("product") );
	m_pDevListCtrl->InsertColumn( lcSerial, wxT("serial") );
	m_pDevListCtrl->InsertColumn( lcState, wxT("state") );
	m_pDevListCtrl->InsertColumn( lcFWVersion, wxT("firmware version") );
	m_pDevListCtrl->InsertColumn( lcKernelDriver, wxT("kernel driver") );
	m_pDevListCtrl->InsertColumn( lcDeviceID, wxT("device ID") );

	m_pLogWindow = new wxTextCtrl(pPanel, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxSUNKEN_BORDER | wxTE_RICH | wxTE_READONLY);
	
	wxBoxSizer* pSizer = new wxBoxSizer(wxVERTICAL);
	pSizer->Add( m_pDevListCtrl, wxSizerFlags(2).Expand() );
	pSizer->Add( m_pLogWindow, wxSizerFlags(1).Expand() );

	BuildList();

	wxTextAttr blueStyle(wxColour(0, 0, 255));
	WriteLogMessage( wxT("Right click on a device entry to get a menu with available options for the selected device\n"), blueStyle );
	WriteLogMessage( wxT("If a menu entry is disabled the underlying feature is not available for the selected device.\n"), blueStyle );
	WriteLogMessage( wxT("\n") );
	WriteLogMessage( wxT("Double click on a list entry to start live acquisiton from this device in a new instance of wxPropView (wxPropView must be locatable via\n"), blueStyle );
	WriteLogMessage( wxT("the systems path or must reside in the same directory as mvDeviceConfigure!)\n"), blueStyle );
	WriteLogMessage( wxT("\n") );
	WriteLogMessage( wxT("Menu entries under 'Action' will be enabled and disabled whenever the currently selected device changes.\n"), blueStyle );
	WriteLogMessage( wxT("\n") );
	WriteLogMessage( wxT("To modify the way log outputs are created select 'Action -> Configure Log Output'.\n"), blueStyle );
	wxRect defaultRect(0, 0, 640, 480);
	SetSizeHints( defaultRect.GetWidth(), defaultRect.GetHeight() );
	pPanel->SetSizer( pSizer );

	// restore previous state
	FramePositionStorage frameStorage(this);
	wxRect rect = frameStorage.restore( defaultRect );
	SetSize( rect );
	UpdateMenu( -1 );

	m_listUpdateTimer.SetOwner( this, teListUpdate );
	m_listUpdateTimer.Start( 500 );
}

//-----------------------------------------------------------------------------
DeviceConfigureFrame::~DeviceConfigureFrame()
//-----------------------------------------------------------------------------
{
	// store the current state of the application
	FramePositionStorage frameStorage(this);
	frameStorage.store();
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::ActivateDeviceInPropView( int deviceIndex )
//-----------------------------------------------------------------------------
{
	wxString commandString(wxT("wxPropView device="));
	commandString.Append(ConvertedString(m_devMgr.getDevice( deviceIndex )->serial.read()));
	commandString.Append( wxT(" live=1") );
	::wxExecute( commandString );
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::BuildList( void )
//-----------------------------------------------------------------------------
{
	typedef map<int, int> IntIntMap;
	typedef pair<int, int> IntIntPair;
	typedef map<string, IntIntMap> String_IntIntMap_Map;
	typedef pair<string, IntIntMap> String_IntIntMap_Pair;
	String_IntIntMap_Map devMap;
	
	m_lastDevMgrChangedCount = m_devMgr.changedCount();
	m_pDevListCtrl->DeleteAllItems();
	unsigned int devCnt = m_devMgr.deviceCount();
	for( unsigned int i=0; i<devCnt; i++ )
	{
		Device* pDev = m_devMgr[i];
		long index = m_pDevListCtrl->InsertItem( i, ConvertedString(pDev->family.read()) );
		pair<String_IntIntMap_Map::iterator, bool> p = devMap.insert( String_IntIntMap_Pair(pDev->family.read(), IntIntMap()) );
		if( !p.second )
		{
			IntIntMap::iterator it = p.first->second.find( pDev->deviceID.read() );
			if( it != p.first->second.end() )
			{
				// mark items that have a conflicting device ID
				m_pDevListCtrl->SetItemTextColour( index, *wxRED );
				m_pDevListCtrl->SetItemTextColour( it->second, *wxRED );
				wxTextAttr redStyle(wxColour(255, 0, 0));
				WriteLogMessage( wxString::Format( wxT("WARNING: The device in row %ld(%s) has the same ID as the device in row %d(%s) belonging to the same device family. This should be resolved.\n"), index, ConvertedString(pDev->serial.read()).c_str(), it->second, ConvertedString(m_devMgr[it->second]->serial.read()).c_str() ), redStyle );
			}
			else
			{
				p.first->second.insert( IntIntPair(pDev->deviceID.read(), index) );
			}
		}
		else
		{
			p.first->second.insert( IntIntPair(pDev->deviceID.read(), index ));
		}
		m_pDevListCtrl->SetItem( index, lcProduct, ConvertedString(pDev->product.read()) );
		m_pDevListCtrl->SetItem( index, lcSerial, ConvertedString(pDev->serial.read()) );
		m_pDevListCtrl->SetItem( index, lcState, ConvertedString(pDev->state.readS()) );
		m_pDevListCtrl->SetItemData( index, i );
		if( pDev->state.read() != dsPresent )
		{
			wxColor col(wxT("LIGHT GREY"));
			m_pDevListCtrl->SetItemBackgroundColour( index, col );
		}
		m_pDevListCtrl->SetItem( index, lcFWVersion, ConvertedString(pDev->firmwareVersion.readS()) );
		string kernelDriverName;
		bool boNewerDriverAvailable = false;
		bool boFeatureSupported = false;
		auto_ptr<DeviceHandler> pHandler(m_deviceHandlerFactory.CreateObject( ConvertedString(pDev->family.read()), pDev ));
		if( pHandler.get() )
		{
			boFeatureSupported = pHandler->SupportsKernelDriverUpdate( boNewerDriverAvailable, kernelDriverName );
		}
		wxString wxKernelDriverName;
		if( kernelDriverName.empty() )
		{
			wxKernelDriverName = wxString(wxT("unsupported"));
		}
		else
		{
			wxKernelDriverName = ConvertedString(kernelDriverName);
		}
		m_pDevListCtrl->SetItem( index, lcKernelDriver, wxKernelDriverName );
		if( boFeatureSupported && boNewerDriverAvailable )
		{
			wxColor col(150, 75, 150);
			wxTextAttr oldKernelDriverStyle(col);
			WriteLogMessage( wxString::Format( wxT("WARNING: The device in row %ld(%s) is not running with the latest kernel driver. To update the kernel driver for each device of that family currently present right click on the device.\n"), index, ConvertedString(pDev->serial.read()).c_str() ), oldKernelDriverStyle );
			m_pDevListCtrl->SetItemBackgroundColour( index, col );
		}
		m_pDevListCtrl->SetItem( index, lcDeviceID, ConvertedString(pDev->deviceID.readS()) );
	}
	m_pDevListCtrl->SetColumnWidth( lcFamily, ( ( devCnt > 0 ) ? wxLIST_AUTOSIZE : wxLIST_AUTOSIZE_USEHEADER ) );
	m_pDevListCtrl->SetColumnWidth( lcProduct, ( ( devCnt > 0 ) ? wxLIST_AUTOSIZE : wxLIST_AUTOSIZE_USEHEADER ) );
	m_pDevListCtrl->SetColumnWidth( lcSerial, ( ( devCnt > 0 ) ? wxLIST_AUTOSIZE : wxLIST_AUTOSIZE_USEHEADER ) );
	m_pDevListCtrl->SetColumnWidth( lcState, ( ( devCnt > 0 ) ? wxLIST_AUTOSIZE : wxLIST_AUTOSIZE_USEHEADER ) );
	m_pDevListCtrl->SetColumnWidth( lcFWVersion, wxLIST_AUTOSIZE_USEHEADER );
	m_pDevListCtrl->SetColumnWidth( lcKernelDriver, wxLIST_AUTOSIZE_USEHEADER );
	m_pDevListCtrl->SetColumnWidth( lcDeviceID, wxLIST_AUTOSIZE_USEHEADER );
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnAbout(wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxBoxSizer *pTopDownSizer;
	wxDialog dlg(this, wxID_ANY, wxString(_("About mvDeviceConfigure")));
	wxIcon icon(mvIcon_xpm);
	dlg.SetIcon( icon );
	pTopDownSizer = new wxBoxSizer(wxVERTICAL);
	wxStaticText *pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("configure tool for %s devices"), COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("(C) 2005 - %s by %s"), CURRENT_YEAR, COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("Version %s"), VERSION_STRING ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxT("Support contact: www.matrix-vision.de") );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("This tool has been written using wxWidgets (www.wxwidgets.org) and was compiled with version %d.%d.%d of this library"), wxMAJOR_VERSION, wxMINOR_VERSION, wxMINOR_VERSION ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxT("The expat wrapper class used internally has been written by Descartes Systems Sciences, Inc.") );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("The complete source of this application can be obtained by contacting %s"), COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	wxButton *pBtnOK = new wxButton(&dlg, wxID_OK, wxT("OK"));
	pBtnOK->SetDefault();
	pTopDownSizer->Add( pBtnOK, 0, wxALL | wxALIGN_RIGHT, 15 );
	dlg.SetSizer( pTopDownSizer );
	pTopDownSizer->Fit( &dlg );
	dlg.ShowModal();
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnConfigureLogOutput( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	LogOutputHandlerDlg dlg(this, m_devMgr, m_debugData);
	dlg.ShowModal();
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnQuit( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	// true is to force the frame to close
	Close(true);
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnSetID( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	SetID( m_pDevListCtrl->GetCurrentItemIndex() );
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnTimer( wxTimerEvent& e )
//-----------------------------------------------------------------------------
{
	switch( e.GetId() )
	{
	case teListUpdate:
		if( m_lastDevMgrChangedCount == m_devMgr.changedCount() )
		{
			return;
		}
		BuildList();
		break;
	}
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnUpdateFirmware( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	UpdateFirmware( m_pDevListCtrl->GetCurrentItemIndex() );
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnUpdateKernelDriver( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	UpdateKernelDriver( m_pDevListCtrl->GetCurrentItemIndex() );
}

//-----------------------------------------------------------------------------
int DeviceConfigureFrame::SetID( int deviceIndex )
//-----------------------------------------------------------------------------
{
	int result = 0;
	if( deviceIndex >= 0 )
	{
		Device* pDev = m_devMgr[deviceIndex];
		if( pDev )
		{
			string devSerial = pDev->serial.read();
			WriteLogMessage( wxString::Format( wxT( "Trying to set new ID for %s. Current ID: %d\n"), ConvertedString(devSerial).c_str(), pDev->deviceID.read() ) );
			long newID = wxGetNumberFromUser(	wxString::Format( wxT("Enter the new ID for device %s.\nMake sure that no other device of the same family already owns this new ID.\n"), ConvertedString(devSerial).c_str() ),
												wxT("New ID:"),
												wxString::Format( wxT("Set new ID for %s(%s)"), ConvertedString(devSerial).c_str(), ConvertedString(pDev->family.read()).c_str() ),
												0,			// start
												didlMin,	// min
												didlMax,	// max
												this );
			if( newID < didlMin )
			{
				result = -4;
				WriteLogMessage( wxT( "Operation canceled by the user.\n" ) );

			}
			else if( newID > didlMax )
			{
				result = -5;
				WriteLogMessage( wxString::Format( wxT( "Invalid ID. Valid IDs range from %d to %d.\n" ), didlMin, didlMax ) );
			}
			else
			{
				Device* pDevByFamily = 0;
				string family = pDev->family.read();
				int devIndex = 0;
				while( ( pDevByFamily = m_devMgr.getDeviceByFamily( family, devIndex++ ) ) != 0 )
				{
					if( ( pDev != pDevByFamily ) && ( pDevByFamily->deviceID.read() == newID ) )
					{
						long posBefore = m_pLogWindow->GetLastPosition();
						WriteLogMessage( wxString::Format( wxT( "WARNING: The new ID(%ld) is already assigned to at least one other device belonging to the same family(%s)(%s)????.\n" ), newID, ConvertedString(pDevByFamily->serial.read()).c_str(), ConvertedString(family).c_str() ) );
						long posAfter = m_pLogWindow->GetLastPosition();
						wxTextAttr redStyle( wxColour( 255, 0, 0 ) );
						m_pLogWindow->SetStyle( posBefore, posAfter, redStyle );
						break;
					}
				}
			}

			if( result == 0 )
			{
				WriteLogMessage( wxString::Format( wxT( "Trying to assign ID %ld to %s.\n" ), newID, ConvertedString(devSerial).c_str() ) );
				int result = pDev->setID(newID);
				if( result == DMR_FEATURE_NOT_AVAILABLE )
				{
					WriteLogMessage( wxT( "This device doesn't support setting the device ID.\n" ) );
				}
				else if( result != DMR_NO_ERROR )
				{
					WriteLogMessage( wxString::Format( wxT( "An error occurred: %s(please refer to the manual for this error code).\n" ), ConvertedString(ImpactAcquireException::getErrorCodeAsString( result )).c_str() ) );
				}
				else
				{
					WriteLogMessage( wxString::Format( wxT( "%s.\n" ), ConvertedString(pDev->HWUpdateResult.readS()).c_str() ) );
				}
			}
		}
		else
		{
			WriteLogMessage(wxString::Format( wxT( "Invalid item selection(index: %d).\n"), deviceIndex ) );
			result = -2;
		}
	}
	else
	{
		wxMessageDialog selectDlg( NULL,
								   wxT("Select a device."),
								   wxT("Error"),
								   wxOK | wxICON_INFORMATION );
		selectDlg.ShowModal();
		result = -1;
	}
	return result;
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::OnUpdateDeviceList( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	WriteLogMessage( wxString(wxT("Updating device list...\n")) );
	m_devMgr.updateDeviceList();
	BuildList();
}

//-----------------------------------------------------------------------------
int DeviceConfigureFrame::UpdateFirmware( int deviceIndex )
//-----------------------------------------------------------------------------
{
	int result = 0;
	if( deviceIndex >= 0 )
	{
		Device* pDev = m_devMgr[deviceIndex];
		if( pDev )
		{
			auto_ptr<DeviceHandler> pHandler(m_deviceHandlerFactory.CreateObject( ConvertedString(pDev->family.read()), pDev ));
			if( pHandler.get() )
			{
				SetCursor( *wxHOURGLASS_CURSOR );
				pHandler->AttachParent( this );
				result = pHandler->UpdateFirmware();
				SetCursor( wxNullCursor );
			}
			else
			{
				WriteLogMessage( wxT("This device doesn't support firmware updates.\n") );
			}
		}
		else
		{
			WriteLogMessage( wxString::Format( wxT("Invalid item selection(index: %d).\n"), deviceIndex ) );
			result = -2;
		}
	}
	else
	{
		wxMessageDialog selectDlg( NULL,
								   wxT("Select a device."),
								   wxT("Error"),
								   wxOK | wxICON_INFORMATION );
		selectDlg.ShowModal();
		result = -1;
	}
	return result;
}

//-----------------------------------------------------------------------------
int DeviceConfigureFrame::UpdateKernelDriver( int deviceIndex )
//-----------------------------------------------------------------------------
{
	int result = 0;
	if( deviceIndex >= 0 )
	{
		Device* pDev = m_devMgr[deviceIndex];
		auto_ptr<DeviceHandler> pHandler(m_deviceHandlerFactory.CreateObject( ConvertedString(pDev->family.read()), pDev ));
		if( pHandler.get() )
		{
			SetCursor( *wxHOURGLASS_CURSOR );
			m_listUpdateTimer.Stop();
			pHandler->AttachParent( this );
			result = pHandler->UpdateKernelDriver();
			m_listUpdateTimer.Start(500);
			SetCursor( wxNullCursor );
		}
		else
		{
			WriteLogMessage( wxString::Format( wxT("Invalid item selection(index: %d).\n"), deviceIndex ) );
			result = -2;
		}
	}
	else
	{
		wxMessageDialog selectDlg( NULL,
								   wxT("Select a device."),
								   wxT("Error"),
								   wxOK | wxICON_INFORMATION );
		selectDlg.ShowModal();
		result = -1;
	}
	return result;
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::UpdateMenu( int deviceIndex )
//-----------------------------------------------------------------------------
{
	Device* pDev = 0;
	if( ( deviceIndex >= 0 ) && ( deviceIndex < static_cast<int>(m_devMgr.deviceCount()) ) )
	{
		pDev = m_devMgr.getDevice( deviceIndex );
	}
	string kernelDriverName;
	bool boNewerDriverAvailable = false;
	bool boFeatureSupported = false;
	bool boFWUpdateSupported = false;
	bool boSetIDSupported = false;
	if( pDev )
	{
		auto_ptr<DeviceHandler> pHandler(m_deviceHandlerFactory.CreateObject( ConvertedString(pDev->family.read()), pDev ));
		if( pHandler.get() )
		{
			boFeatureSupported = pHandler->SupportsKernelDriverUpdate( boNewerDriverAvailable, kernelDriverName );
			boFWUpdateSupported = pHandler->SupportsFirmwareUpdate();
			boSetIDSupported = pHandler->SupportsSetID();
		}
	}
	m_pMIActionSetID->Enable( boSetIDSupported );
	m_pMIActionUpdateFW->Enable( boFWUpdateSupported );
	m_pMIActionUpdateKernelDriver->Enable( boFeatureSupported && boNewerDriverAvailable );
}

//-----------------------------------------------------------------------------
void DeviceConfigureFrame::WriteLogMessage( const wxString& msg, const wxTextAttr& style /* = wxTextAttr(wxColour(0, 0, 0)) */ )
//-----------------------------------------------------------------------------
{
	if( m_pLogWindow )
	{
		long posBefore = m_pLogWindow->GetLastPosition();
		m_pLogWindow->WriteText( msg );
		long posAfter = m_pLogWindow->GetLastPosition();
		m_pLogWindow->SetStyle( posBefore, posAfter, style );
	}
}
