//-----------------------------------------------------------------------------
#include <apps/Common/wxAbstraction.h>
#include "DeviceConfigureFrame.h"
#include "DeviceHandlerBlueDevice.h"
#include <wx/ffile.h>
#include <wx/filedlg.h>
#include <wx/filefn.h>
#include <wx/msgdlg.h>

//-----------------------------------------------------------------------------
DeviceHandlerBlueDevice::DeviceHandlerBlueDevice( mvIMPACT::acquire::Device* pDev ) : DeviceHandler(pDev), product_(pgUnknown)
//-----------------------------------------------------------------------------
{
	ConvertedString product(pDev_->product.read());
	if( product.Find( wxT("mvBlueCOUGAR-P") ) != wxNOT_FOUND )
	{
		product_ = pgBlueCOUGAR_P;
	}
	else if( product.Find( wxT("mvBlueCOUGAR-S") ) != wxNOT_FOUND )
	{
		product_ = pgBlueCOUGAR_S;
	}
	else if( product.Find( wxT("mvBlueLYNX-M7") ) != wxNOT_FOUND )
	{
		product_ = pgBlueLYNX_M7;
	}

	switch( product_ )
	{
	case pgBlueCOUGAR_P:
		firmwareUpdateFileName_ = wxString(wxT("mvBlueCOUGAR-P_Update.tgz"));
		productStringForFirmwareUpdateCheck_ = wxString(wxT("mvBlueCOUGAR-P"));
		break;
	case pgBlueCOUGAR_S:
		firmwareUpdateFileName_ = product;
		if( pDev->firmwareVersion.read() >= 0x10100 )
		{
			// this is HW revision 1, which means it needs a different firmware
			firmwareUpdateFileName_.Append( wxT("_R001") );
		}
		firmwareUpdateFileName_.Append( wxT("_Update.fpg") );
		productStringForFirmwareUpdateCheck_ = product;
		break;
	case pgBlueLYNX_M7:
		firmwareUpdateFileName_ = wxString(wxT("mvBlueLYNX-M7_Update.tgz"));
		productStringForFirmwareUpdateCheck_ = wxString(wxT("mvBlueLYNX-M7"));
		break;
	default:
		break;
	}

	static const wxString MVIMPACT_ACQUIRE_DIR(wxT("MVIMPACT_ACQUIRE_DIR"));
	if( !::wxGetEnv( MVIMPACT_ACQUIRE_DIR, &firmwareUpdateFolder_ ) )
	{
		pParent_->WriteLogMessage( wxT("Can't install new firmware automatically as a crucial environment variable(MVIMPACT_ACQUIRE_DIR) is missing...\nPlease select the folder containing the files manually.\n"), wxTextAttr(wxColour(255, 0, 0)) );
	}
	else
	{
		firmwareUpdateFolder_.append( wxT("/FirmwareUpdates/mvBlueCOUGAR") );
	}
}

//-----------------------------------------------------------------------------
bool DeviceHandlerBlueDevice::SupportsFirmwareUpdate( void ) const
//-----------------------------------------------------------------------------
{
	return !firmwareUpdateFileName_.IsEmpty();
}

//-----------------------------------------------------------------------------
int DeviceHandlerBlueDevice::UpdateLYNXAndCOUGAR_PDevices( const wxString& updateName, const wxString& updateFileName, const wxString& /* fileExtension */, const wxString& filenameValidationString )
//-----------------------------------------------------------------------------
{
	ConvertedString serial(pDev_->serial.read());
	wxMessageDialog closeDeviceDlg( pParent_,
		wxString::Format( wxT("Please close all other applications using device %s before proceeding\n"), serial.c_str() ),
		wxT("Firmware update"),
		wxCANCEL | wxOK | wxICON_INFORMATION );
	if( closeDeviceDlg.ShowModal() != wxID_OK )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("%s update canceled for device %s.\n"), updateName.c_str(), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}

	wxFileDialog fileDlg(pParent_, wxT("Please select the update file"), wxT(""), updateFileName, wxString::Format( wxT("%s update file (%s)|%s"), ConvertedString(pDev_->product.read()).c_str(), updateFileName.c_str(), updateFileName.c_str() ), wxOPEN | wxFILE_MUST_EXIST);
	if( fileDlg.ShowModal() != wxID_OK )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("%s update canceled for device %s.\n"), updateName.c_str(), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}

	if( fileDlg.GetFilename().Find( filenameValidationString.c_str() ) != 0 )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("The file %s is not meant for updating device %s.\n"), fileDlg.GetFilename().c_str(), serial.c_str() ), wxTextAttr(wxColour(255, 0, 0)) );
		return urInvalidFileSelection;
	}

	wxMessageDialog AreYouSureDlg( pParent_,
		wxString::Format( wxT("Are you sure you want to update the %s of device %s?\nThis might take several minutes during which the application will not react.\nPlease be patient.\n"), updateName.c_str(), serial.c_str() ),
		wxString::Format( wxT("%s update"), updateName.c_str() ),
		wxNO_DEFAULT | wxYES_NO | wxICON_INFORMATION );

	if( AreYouSureDlg.ShowModal() != wxID_YES )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("%s update canceled for device %s.\n"), updateName.c_str(), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}

	try
	{
		pDev_->open();
	}
	catch( const ImpactAcquireException& e )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Failed to open device %s(%s, %s).\n"), 
			ConvertedString(serial).c_str(),
			ConvertedString(e.getErrorString()).c_str(),
			ConvertedString(e.getErrorCodeAsString()).c_str() ) );
		return urDeviceAccessError;
	}

	try
	{
		DeviceComponentLocator locator(pDev_, dltSystemSettings);
		PropertyS sourceFileName(locator.findComponent( "SourceFileName" ));
		PropertyS destinationFileName(locator.findComponent( "DestinationFileName" ));
		PropertyI transferMode(locator.findComponent( "TransferMode" ));
		Method upload(locator.findComponent( "DoFileUpload@i" ));
		Method download(locator.findComponent( "DoFileDownload@i" ));
		Method install(locator.findComponent( "DoInstallFile@i" ));
		PropertyS lastResult(locator.findComponent( "LastResult" ));

		transferMode.writeS( "Binary" );

		wxString directory(fileDlg.GetDirectory());
		pParent_->WriteLogMessage( wxT("Installing updates. This will take some minutes. During this time the application will not react.\n") );

		// upload the file
		sourceFileName.write( std::string(wxConvCurrent->cWX2MB(fileDlg.GetPath().c_str())) );
		destinationFileName.write( std::string(wxConvCurrent->cWX2MB(fileDlg.GetFilename().c_str())) );
		int functionResult = DMR_NO_ERROR;
		if( ( functionResult = upload.call( std::string(wxConvCurrent->cWX2MB( wxT("") )) ) ) == DMR_NO_ERROR )
		{
			// install the file
			if( ( functionResult = install.call( std::string(wxConvCurrent->cWX2MB( wxT("") )) ) ) == DMR_NO_ERROR )
			{
				pParent_->WriteLogMessage( wxString::Format( wxT("Result of the installation of %s: %s\n"), fileDlg.GetPath().c_str(), ConvertedString(lastResult.read()).c_str() ) );

				// download the installation log file
				wxString logFileName(fileDlg.GetFilename());
				logFileName.append( wxT(".install.log") );
				wxString fullLogFilePath( wxT("/tmp/") );
				fullLogFilePath.append( logFileName );
				sourceFileName.write( std::string(wxConvCurrent->cWX2MB(fullLogFilePath.c_str())) );
				wxString logFileDestinationName(directory);
				logFileDestinationName.append( wxT("/") );
				logFileDestinationName.append( logFileName );
				destinationFileName.write( std::string(wxConvCurrent->cWX2MB(logFileDestinationName.c_str())) );
				download.call( std::string(wxConvCurrent->cWX2MB( wxT("") )) );
				pParent_->WriteLogMessage( wxString::Format( wxT("Result of the downloading the installation log file %s to %s: %s\n"), fileDlg.GetFilename().c_str(), logFileDestinationName.c_str(), ConvertedString(lastResult.read()).c_str() ) );

				// try to display and remove the *.log-file
				{
					wxFFile logFile(logFileDestinationName.c_str(), wxT("r"));
					if( logFile.IsOpened() )
					{
						wxString logData;
						logFile.ReadAll( &logData );
						pParent_->WriteLogMessage( logData );
					}
					else
					{
						pParent_->WriteLogMessage( wxString::Format( wxT("ERROR! Couldn't open temporary file %s for reading.\n"), logFileDestinationName.c_str() ), wxTextAttr(wxColour(255, 0, 0)) );
					}
				}

				if( !::wxRemoveFile( logFileDestinationName ) )
				{
					pParent_->WriteLogMessage( wxString::Format( wxT("ERROR! Couldn't delete temporary file %s. Must be deleted manually.\n"), logFileDestinationName.c_str() ), wxTextAttr(wxColour(255, 0, 0)) );
				}

				wxMessageDialog infoDlg( NULL,
					wxString::Format( wxT("The %s has been updated. Please close this and all other applications\nusing this device and disconnect the device from the power supply.\nAfter rebooting the new %s will be active.\n"), updateName.c_str(), updateName.c_str() ),
					wxT("information"),
					wxOK | wxICON_INFORMATION );
				infoDlg.ShowModal();
			}
			else
			{
				pParent_->WriteLogMessage( wxString::Format( wxT("Failed to install %s: %d(%s) %s\n"), 
					fileDlg.GetFilename().c_str(),
					functionResult,
					ConvertedString(ImpactAcquireException::getErrorCodeAsString( functionResult )).c_str(),
					ConvertedString(lastResult.read()).c_str() ) );
			}
		}
		else
		{
			pParent_->WriteLogMessage( wxString::Format( wxT("Failed to upload %s: %d(%s) %s\n"), 
				fileDlg.GetFilename().c_str(),
				functionResult,
				ConvertedString(ImpactAcquireException::getErrorCodeAsString( functionResult )).c_str(),
				ConvertedString(lastResult.read()).c_str() ) );
		}
	}
	catch( const ImpactAcquireException& e )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Failed to update device %s(%s, %s).\n"), 
			ConvertedString(serial).c_str(),
			ConvertedString(e.getErrorString()).c_str(),
			ConvertedString(e.getErrorCodeAsString()).c_str() ) );
		pDev_->close();
		return urDeviceAccessError;
	}
	pDev_->close();
	return urOperationSuccessful;
}

//-----------------------------------------------------------------------------
int DeviceHandlerBlueDevice::UploadFile( const wxString& fullPath, const wxString& descriptionFile )
//-----------------------------------------------------------------------------
{
	int result = urOperationSuccessful;
	ConvertedString serial(pDev_->serial.read());
	try
	{
		ComponentLocator locatorDevice(pDev_->hDev());
		PropertyI descriptionToUse(locatorDevice.findComponent( "DescriptionToUse" ));
		descriptionToUse.writeS( "CustomFile" );
		PropertyS customDescriptionFilename(locatorDevice.findComponent( "CustomDescriptionFilename" ));
		customDescriptionFilename.write( std::string(wxConvCurrent->cWX2MB(descriptionFile.c_str())) );
		pDev_->open();
	}
	catch( const ImpactAcquireException& e )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Failed to open device %s(%s, %s).\n"), 
			ConvertedString(serial).c_str(),
			ConvertedString(e.getErrorString()).c_str(),
			ConvertedString(e.getErrorCodeAsString()).c_str() ) );
		return urDeviceAccessError;
	}

	try
	{
		DeviceComponentLocator locator(pDev_, dltSystemSettings);
		PropertyS sourceFileName(locator.findComponent( "SourceFileName" ));
		PropertyI transferMode(locator.findComponent( "TransferMode" ));
		Method upload(locator.findComponent( "DoFileUpload@i" ));
		PropertyS lastResult(locator.findComponent( "LastResult" ));

		transferMode.writeS( "Binary" );
		pParent_->WriteLogMessage( wxString::Format( wxT("Installing %s. This will take some minutes. During this time the application will not react.\n"), fullPath.c_str() ) );

		// upload the file
		sourceFileName.write( std::string(wxConvCurrent->cWX2MB(fullPath.c_str())) );
		upload.call( std::string(wxConvCurrent->cWX2MB( wxT("") )) );
		pParent_->WriteLogMessage( wxString::Format( wxT("Result of the installation of %s: %s\n"), fullPath.c_str(), ConvertedString(lastResult.read()).c_str() ) );
	}
	catch( const ImpactAcquireException& e )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Failed to update device %s(%s, %s).\n"), 
			ConvertedString(serial).c_str(),
			ConvertedString(e.getErrorString()).c_str(),
			ConvertedString(e.getErrorCodeAsString()).c_str() ) );
		result = urDeviceAccessError;
	}
	pDev_->close();
	return result;
}

//-----------------------------------------------------------------------------
int DeviceHandlerBlueDevice::UpdateCOUGAR_SDevices( void )
//-----------------------------------------------------------------------------
{
	ConvertedString serial(pDev_->serial.read());
	wxMessageDialog closeDeviceDlg( pParent_,
		wxString::Format( wxT("Please close all other applications using device %s before proceeding\n"), serial.c_str() ),
		wxT("Firmware update"),
		wxCANCEL | wxOK | wxICON_INFORMATION );
	if( closeDeviceDlg.ShowModal() != wxID_OK )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Firmware update canceled for device %s.\n"), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}

	wxFileDialog fileDlgFirmware(pParent_, wxT("Please select the firmware file"), firmwareUpdateFolder_, firmwareUpdateFileName_, wxString::Format( wxT("%s firmware file (%s)|%s"), ConvertedString(pDev_->product.read()).c_str(), firmwareUpdateFileName_.c_str(), firmwareUpdateFileName_.c_str() ), wxOPEN | wxFILE_MUST_EXIST);
	if( fileDlgFirmware.ShowModal() != wxID_OK )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Firmware update canceled for device %s.\n"), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}

	if( fileDlgFirmware.GetFilename().Find( productStringForFirmwareUpdateCheck_.c_str() ) != 0 )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("The file %s is not meant for updating device %s.\n"), fileDlgFirmware.GetFilename().c_str(), serial.c_str() ), wxTextAttr(wxColour(255, 0, 0)) );
		return urInvalidFileSelection;
	}

	wxFileDialog fileDlgDescriptionFile(pParent_, wxT("Please select the GenICam description file that came with the firmware"), firmwareUpdateFolder_, wxT(""), wxString::Format( wxT("%s GenICam description file (MATRIXVISION_mvBlueCOUGAR-S_GigE*.zip)|MATRIXVISION_mvBlueCOUGAR-S_GigE*.zip"), ConvertedString(pDev_->product.read()).c_str() ), wxOPEN | wxFILE_MUST_EXIST);
	if( fileDlgDescriptionFile.ShowModal() != wxID_OK )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Firmware update canceled for device %s.\n"), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}

	if( fileDlgDescriptionFile.GetFilename().Find( wxT("MATRIXVISION_mvBlueCOUGAR-S_GigE") ) != 0 )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("The file %s is not meant for updating device %s.\n"), fileDlgDescriptionFile.GetFilename().c_str(), serial.c_str() ), wxTextAttr(wxColour(255, 0, 0)) );
		return urInvalidFileSelection;
	}

	wxMessageDialog AreYouSureDlg( pParent_,
		wxString::Format( wxT("Are you sure you want to update the firmware and description file of device %s?\nThis might take several minutes during which the application will not react.\nPlease be patient.\n"), serial.c_str() ),
		wxT("Firmware update"),
		wxNO_DEFAULT | wxYES_NO | wxICON_INFORMATION );

	if( AreYouSureDlg.ShowModal() != wxID_YES )
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Firmware update canceled for device %s.\n"), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}

	int result = UploadFile( fileDlgDescriptionFile.GetPath(), fileDlgDescriptionFile.GetPath() );
	if( result != urOperationSuccessful )
	{
		return result;
	}
	result = UploadFile( fileDlgFirmware.GetPath(), fileDlgDescriptionFile.GetPath() );
	if( result == urOperationSuccessful )
	{
		wxMessageDialog infoDlg( NULL,
			wxT("The Firmware has been updated. Please close this and all other applications\nusing this device and disconnect the device from the power supply.\nAfter rebooting the new Firmware will be active.\n"),
			wxT("information"),
			wxOK | wxICON_INFORMATION );
		infoDlg.ShowModal();
	}
	return result;
}

//-----------------------------------------------------------------------------
int DeviceHandlerBlueDevice::UpdateFirmware( void )
//-----------------------------------------------------------------------------
{
	if( product_ == pgBlueCOUGAR_S )
	{
		return UpdateCOUGAR_SDevices();
	}
	else
	{
		return UpdateLYNXAndCOUGAR_PDevices( wxString(wxT("Firmware")), firmwareUpdateFileName_, wxString(wxT(".tgz")), productStringForFirmwareUpdateCheck_ );
	}
}
