//-----------------------------------------------------------------------------
#include <apps/Common/wxAbstraction.h>
#include "CaptureThread.h"
#include "PropViewFrame.h"
#include <wx/event.h>

//-----------------------------------------------------------------------------
class TimerLite
//-----------------------------------------------------------------------------
{
	clock_t			m_endClocks; 
	clock_t			Delay2Clocks( unsigned long nextDelay_ms ) { return (nextDelay_ms*CLOCKS_PER_SEC)/1000;}
public:
	TimerLite() :	m_endClocks(0) {;}
	TimerLite( unsigned long nextDelay_ms ) { Restart( nextDelay_ms );}
	void	Restart( unsigned long nextDelay_ms ){ m_endClocks = clock() + Delay2Clocks( nextDelay_ms ); } 
	bool	IsExpired( unsigned long nextDelay_ms )
	{	clock_t curClocks = clock();
		if( curClocks > m_endClocks ){ m_endClocks = curClocks + Delay2Clocks( nextDelay_ms ); return true; }
		else return false;
	}
};

using namespace std;
using namespace mvIMPACT::acquire;

unsigned int CaptureThread::m_activeInstance = 0;
unsigned int CaptureThread::m_instanceCnt = 0;

//-----------------------------------------------------------------------------
CaptureThread::CaptureThread( FunctionInterface* pFuncInterface, Device* pDev, wxWindow* pParentWindow ) : 
	wxThread(wxTHREAD_JOINABLE), m_boLive(false), m_boRecord(false), m_currentRequest(INVALID_ID),
	m_discardedImages(0), m_instanceNr(m_instanceCnt++), m_nextPendingRequest(INVALID_ID),
	m_parentWindowID(pParentWindow->GetId()), m_pDev(pDev), m_pFuncInterface(pFuncInterface), 
	m_pImageRequestControl(new ImageRequestControl(pDev)),
	m_pParentWindowEventHandler(pParentWindow->GetEventHandler()), m_currentSequenceIndexToDisplay(0),
	m_currentCountToRecord(0)
//-----------------------------------------------------------------------------
{

}

//-----------------------------------------------------------------------------
CaptureThread::~CaptureThread()
//-----------------------------------------------------------------------------
{ 
	if( m_nextPendingRequest != INVALID_ID )
	{
		m_pFuncInterface->imageRequestUnlock( m_nextPendingRequest );
		m_nextPendingRequest = INVALID_ID;
	}

	if( m_currentRequest != INVALID_ID )
	{
		m_pFuncInterface->imageRequestUnlock( m_currentRequest );
		m_currentRequest = INVALID_ID;
	}
	delete m_pImageRequestControl;
}

//-----------------------------------------------------------------------------
TRequestResult CaptureThread::GetImageData( ImageBufferDesc* pImgBufDesc, RequestInfoData* pReqInfo, wxString& pixelFormat, bool boFillInfoVector, std::vector<wxString>& infoStrings ) const
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	if( m_nextPendingRequest != INVALID_ID )
	{
		if( m_currentRequest != INVALID_ID )
		{
			UnlockRequest( m_currentRequest );
		}
		m_currentRequest = m_nextPendingRequest;
		m_nextPendingRequest = INVALID_ID;
	}

	if( m_currentRequest != INVALID_ID )
	{
		const Request* pReq = m_pFuncInterface->getRequest( m_currentRequest );
		if( pReq )
		{
			ImageBufferDesc nbd(pReq->getImageBufferDesc());
			*pImgBufDesc = nbd;
			//*pImgBufDesc = ImageBufferDesc(ib); // doesn't work with GCC...
			pReqInfo->cameraOutputUsed = ( pReq->infoCameraOutputUsed.isValid() ) ? pReq->infoCameraOutputUsed.read() : coUndefined;
			pReqInfo->exposeStart_us = pReq->infoExposeStart_us.read();
			pReqInfo->exposeTime_us = pReq->infoExposeTime_us.read();
			pReqInfo->frameID = pReq->infoFrameID.read();
			pReqInfo->frameNr = pReq->infoFrameNr.read();
			pReqInfo->gain_dB = pReq->infoGain_dB.read();
			pReqInfo->lineCounter = ( pReq->infoLineCounter.isValid() ) ? pReq->infoLineCounter.read() : -1;
			pReqInfo->timeStamp_us = pReq->infoTimeStamp_us.read();
			pReqInfo->transferDelay_us = ( pReq->infoTransferDelay_us.isValid() ) ? pReq->infoTransferDelay_us.read() : -1;
			pReqInfo->videoChannel = (pReq->infoVideoChannel.isValid() ) ? pReq->infoVideoChannel.read() : -1;
			pixelFormat = ConvertedString(pReq->imagePixelFormat.readS());
			if( boFillInfoVector )
			{
				ComponentIterator it(pReq->getInfoIterator());
				while( it.isValid() )
				{
					if( it.isProp() && it.isVisible() )
					{
						Property prop(it);
						infoStrings.push_back( wxString::Format( wxT("%s: %s"), ConvertedString(prop.name()).c_str(), ConvertedString(prop.readS()).c_str() ) );
					}
					++it;
				}
			}
			return pReq->requestResult.read();
		}
	}
	else
	{
		ImageBufferDesc nbd(1);
		*pImgBufDesc = nbd;
		//*pImgBufDesc = ImageBufferDesc(ib); // doesn't work with GCC...
		pixelFormat = wxT("unknown");
		memset( pReqInfo, 0, sizeof(RequestInfoData) );
		return rrOK;
	}
	return rrError;
}

//-----------------------------------------------------------------------------
size_t CaptureThread::DisplaySequenceRequest( size_t nr )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	m_currentSequenceIndexToDisplay = ( nr < m_vSequence.size() ) ? nr : m_vSequence.size() - 1;	
	int req = m_vSequence[m_currentSequenceIndexToDisplay];
	if( req != m_currentRequest )
	{
		if( m_nextPendingRequest != INVALID_ID )
		{
			UnlockRequest( m_nextPendingRequest );
		}
		m_nextPendingRequest = req;
	}

	return m_currentSequenceIndexToDisplay;
}

//-----------------------------------------------------------------------------
size_t CaptureThread::DisplaySequenceRequestNext( void )
//-----------------------------------------------------------------------------
{
	return DisplaySequenceRequest( m_currentSequenceIndexToDisplay + 1 );
}

//-----------------------------------------------------------------------------
size_t CaptureThread::DisplaySequenceRequestPrev( void )
//-----------------------------------------------------------------------------
{
	return DisplaySequenceRequest( ( m_currentSequenceIndexToDisplay != 0 ) ? ( m_currentSequenceIndexToDisplay - 1 ) : 0 );
}

//-----------------------------------------------------------------------------
void* CaptureThread::Entry()
//-----------------------------------------------------------------------------
{
	const unsigned long waitBeforeBreak = 60;
	TimerLite forceBreakTimer( waitBeforeBreak );
	while( !TestDestroy() )
	{
		int requestNr;
		if( m_boLive )
		{
			RequestImages();
		}
		if( ( requestNr = m_pFuncInterface->imageRequestWaitFor( 0, 0 ) ) < 0 )
		{
			if( ( requestNr = m_pFuncInterface->imageRequestWaitFor( 200, 0 ) ) >= 0 )
			{
				forceBreakTimer.Restart( waitBeforeBreak );
			}
		}
		else if( forceBreakTimer.IsExpired( waitBeforeBreak ) )
		{
			Sleep(10);
		}
		if( requestNr >= 0 )
		{
			wxCriticalSectionLocker locker(m_critSect);
			if( m_pFuncInterface->isRequestNrValid( requestNr ) )
			{
				const Request* pReq = m_pFuncInterface->getRequest( requestNr );
				if( pReq )
				{
					if( m_activeInstance == m_instanceNr )
					{
						TRequestResult rr = pReq->requestResult.read();
						m_lastRequestResult = ConvertedString(pReq->requestResult.readS().c_str());
						if( rr == rrOK )
						{
							RequestInfoData* pReqInfo = new RequestInfoData();
							pReqInfo->cameraOutputUsed = ( pReq->infoCameraOutputUsed.isValid() ) ? pReq->infoCameraOutputUsed.read() : coUndefined;
							pReqInfo->exposeStart_us = pReq->infoExposeStart_us.read();
							pReqInfo->exposeTime_us = pReq->infoExposeTime_us.read();
							pReqInfo->frameID = pReq->infoFrameID.read();
							pReqInfo->frameNr = pReq->infoFrameNr.read();
							pReqInfo->gain_dB = pReq->infoGain_dB.read();
							pReqInfo->lineCounter = ( pReq->infoLineCounter.isValid() ) ? pReq->infoLineCounter.read() : -1;
							pReqInfo->timeStamp_us = pReq->infoTimeStamp_us.read();
							pReqInfo->transferDelay_us = ( pReq->infoTransferDelay_us.isValid() ) ? pReq->infoTransferDelay_us.read() : -1;
							pReqInfo->videoChannel = (pReq->infoVideoChannel.isValid() ) ? pReq->infoVideoChannel.read() : -1;
							wxCommandEvent eInfo(imageInfoEvent, m_parentWindowID);
							eInfo.SetClientData( pReqInfo );
							wxPostEvent( m_pParentWindowEventHandler, eInfo );
							if( m_boRecord )
							{
								m_vSequence.push_back( requestNr );
							}
							if( m_nextPendingRequest != INVALID_ID )
							{
								UnlockRequest( m_nextPendingRequest );
								++m_discardedImages;
								if( m_SkippedImageCount++ == 0 )
								{
									wxCommandEvent e( imageSkippedEvent, m_parentWindowID );
									wxPostEvent( m_pParentWindowEventHandler, e );
								}
								m_nextPendingRequest = requestNr;
							}
							else
							{
								m_nextPendingRequest = requestNr;
								wxCommandEvent e(imageReadyEvent, m_parentWindowID);
								wxPostEvent( m_pParentWindowEventHandler, e );
							}
						}
						else
						{
							if( rr & rrUnprocessibleRequest )
							{
								m_lastErrorMessage = ConvertedString(pReq->requestResult.readS());
							}
							m_pFuncInterface->imageRequestUnlock( requestNr );
							if( GetLiveMode() )
							{
								// Livemode is stopped depending on requestresult, TRequestResult
								if( rr & rrUnprocessibleRequest )
								{
									SetLiveMode( false );
									m_pFuncInterface->imageRequestReset( 0, 0 );
									wxCommandEvent e(liveModeAborted, m_parentWindowID);
									e.SetString(m_lastErrorMessage);
									wxPostEvent( m_pParentWindowEventHandler, e );
									if( m_currentCountToRecord )
									{
										m_currentCountToRecord = 1;
									}
								}
							}
						}

						if( m_currentCountToRecord && m_boRecord )
						{
							--m_currentCountToRecord;
							if( m_currentCountToRecord == 0 )
							{
								wxCommandEvent e( sequenceReadyEvent, m_parentWindowID );
								wxPostEvent( m_pParentWindowEventHandler, e );
								SetLiveMode( false );
							}
						}
					}
					else
					{
						m_pFuncInterface->imageRequestUnlock( requestNr );
						++m_discardedImages;
					}
				}
			}
		}
	}
	return 0;
}

//-----------------------------------------------------------------------------
int CaptureThread::ExchangeLastSnapResult( int newResult ) const
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	int oldval = m_currentRequest;
	m_currentRequest = newResult;
	return oldval;
}

//-----------------------------------------------------------------------------
wxString CaptureThread::GetLastRequestResult( void ) const
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	return m_lastRequestResult;
}

//-----------------------------------------------------------------------------
int CaptureThread::GetLastSnapResult( void ) const
//-----------------------------------------------------------------------------
{
	return ExchangeLastSnapResult( INVALID_ID );
}

//-----------------------------------------------------------------------------
void CaptureThread::SetActive( void )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	m_activeInstance = m_instanceNr;
}

//-----------------------------------------------------------------------------
void CaptureThread::UnlockRequest( int nr ) const
//-----------------------------------------------------------------------------
{
	// unlock of request only when not in sequence
	// ( a request usage counter would be better )
	vector<int>::const_iterator it = m_vSequence.begin();
	for( it = m_vSequence.begin(); it != m_vSequence.end(); it++ )
	{
		if( *it == nr )
		{
			break;
		}
	}
	if( it == m_vSequence.end() )
	{
		m_pFuncInterface->imageRequestUnlock( nr );
	}
}

//-----------------------------------------------------------------------------
void CaptureThread::FreeSequence( void )
//-----------------------------------------------------------------------------
{
	while( !m_vSequence.empty() )
	{
		int nr =  m_vSequence.back();
		if( nr != m_currentRequest )
		{
			m_pFuncInterface->imageRequestUnlock( nr );
		}
		m_vSequence.pop_back();
	}
	m_currentSequenceIndexToDisplay = 0;
}

//-----------------------------------------------------------------------------
int CaptureThread::RequestSingle( void )
//-----------------------------------------------------------------------------
{
	if( !m_boRecord )
	{
		FreeSequence();
	}

	int result = m_pFuncInterface->imageRequestSingle( m_pImageRequestControl );
	if( m_boRecord && ( result < 0 ) && CheckSequenceRestart() )
	{
		SetRecordMode( true );
		RequestSingle();
	}

	return result;
}

//-----------------------------------------------------------------------------
int CaptureThread::SetRecordMode( bool boOn )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);

	if( boOn )
	{
		m_currentSequenceIndexToDisplay = 0;
		m_pFuncInterface->imageRequestReset( 0, 0 );
		if( m_nextPendingRequest != INVALID_ID )
		{
			m_pFuncInterface->imageRequestUnlock( m_nextPendingRequest );
			m_nextPendingRequest = INVALID_ID;
		}
		FreeSequence();
		m_currentCountToRecord = m_pFuncInterface->requestCount();
	}
	m_boRecord = boOn;

	return m_boRecord;
}

//-----------------------------------------------------------------------------
bool CaptureThread::CheckSequenceRestart( void ) const
//-----------------------------------------------------------------------------
{
	bool result = false;
	wxMessageDialog dlg( NULL,
		wxString::Format( wxT("Restarts the image recording. All current recorded images (%lu) will be lost.\n"), (unsigned long int) GetSequenceImageCnt() ),
		wxT("Warning"),
		wxNO_DEFAULT | wxYES_NO | wxICON_INFORMATION );

	if( dlg.ShowModal() == wxID_YES )
	{
		result = true;
	}

	return result;
}

//-----------------------------------------------------------------------------
int CaptureThread::SetLiveMode( bool boOn )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);

	if( boOn )
	{
		m_pFuncInterface->imageRequestReset( 0, 0 );
		if( m_nextPendingRequest != INVALID_ID )
		{
			m_pFuncInterface->imageRequestUnlock( m_nextPendingRequest );
			m_nextPendingRequest = INVALID_ID;
		}

		if( !m_boRecord )
		{
			FreeSequence();
		}

		m_discardedImages = 0;
		m_SkippedImageCount = 0;
		int result = RequestImages();
		if( ( result == DMR_NO_ERROR ) || ( result == DEV_NO_FREE_REQUEST_AVAILABLE ) )
		{
			m_boLive = boOn;
		}
		else
		{
			if( m_boRecord )
			{
				if( CheckSequenceRestart() )
				{
					SetRecordMode( true );
					SetLiveMode( boOn );
				}
			}
			else
			{
				wxMessageDialog dlg( NULL, wxString::Format( wxT("Live not started(Reason: %s)! More information can be found in the *.log-file or the debug output.\n"), ConvertedString(ImpactAcquireException::getErrorCodeAsString( result )).c_str() ), wxT("Error"), wxOK | wxICON_WARNING );
				dlg.ShowModal();
			}
		}
	}
	else
	{
		m_boLive = boOn;
	}

	return m_boLive;
}

//-----------------------------------------------------------------------------
int CaptureThread::RequestImages( void )
//-----------------------------------------------------------------------------
{
	int requestResult = DMR_NO_ERROR;
	do
	{
		if( ( requestResult = m_pFuncInterface->imageRequestSingle( m_pImageRequestControl ) ) != DMR_NO_ERROR )
		{
			break;
		}
	} while( true );
	return requestResult;
}
