#include "PropViewFrame.h"
#include <wxPropGrid/Include/propgrid.h>
#include <wx/config.h>
#include <wx/file.h>
#include <wx/filename.h>
#include <wx/notebook.h>
#include <wx/spinctrl.h>
#include <wx/splitter.h>
#include <wx/tooltip.h>
#include <apps/Common/mvIcon.xpm>
#include "CaptureThread.h"
#include "HistogramCanvas.h"
#include "ImageCanvas.h"
#include "InfoPlotCanvas.h"
#include <apps/Common/Info.h>
#include "LineProfileCanvas.h"
#include "PropData.h"
#include <limits>
#include <memory>
#include "icons.h"

using namespace mvIMPACT::acquire;
using namespace std;

DEFINE_EVENT_TYPE(imageReadyEvent)
DEFINE_EVENT_TYPE(imageSkippedEvent)
DEFINE_EVENT_TYPE(toggleDisplayArea)
DEFINE_EVENT_TYPE(liveModeAborted)
DEFINE_EVENT_TYPE(eventNotified)
DEFINE_EVENT_TYPE(sequenceReadyEvent)
DEFINE_EVENT_TYPE(imageInfoEvent)

//=============================================================================
//================= Implementation MyApp ======================================
//=============================================================================
//-----------------------------------------------------------------------------
class MyApp : public wxApp
//-----------------------------------------------------------------------------
{
public:
	virtual bool OnInit( void )
	{
		wxImage::AddHandler(new wxPNGHandler);
		// Create the main application window
		wxString cmdline;
		for( int i=1; i<argc; i++ )
		{
			cmdline.Append( ConvertedString(argv[i]) );
			cmdline.Append( wxT(' ') );
		}

		PropViewFrame *pFrame = new PropViewFrame( wxString::Format( wxT("wxPropView(%s)"), VERSION_STRING ), wxDefaultPosition, wxDefaultSize, cmdline );
		pFrame->Show( true );
		SetTopWindow( pFrame );
		// success: wxApp::OnRun() will be called which will enter the main message
		// loop and the application will run. If we returned false here, the
		// application would exit immediately.
		return true;
	}
};

// Create a new application object: this macro will allow wxWidgets to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also declares the accessor function
// wxGetApp() which will return the reference of the right type (i.e. MyApp and
// not wxApp)
IMPLEMENT_APP(MyApp)

//=============================================================================
//================= Implementation FramePositionStorage =======================
//=============================================================================
//-----------------------------------------------------------------------------
class FramePositionStorage
//-----------------------------------------------------------------------------
{
	wxConfigBase* m_pConfig;
	wxWindow* m_pWin;
public:
	FramePositionStorage( wxWindow* pWin ) : m_pConfig(wxConfigBase::Get()), m_pWin(pWin) {}
	void store( void ) const
	{
		int Height, Width, XPos, YPos;
		m_pWin->GetSize( &Width, &Height );
		m_pWin->GetPosition( &XPos, &YPos );
		m_pConfig->Write( wxT("/MainFrame/h"), Height );
		m_pConfig->Write( wxT("/MainFrame/w"), Width );
		m_pConfig->Write( wxT("/MainFrame/x"), XPos );
		m_pConfig->Write( wxT("/MainFrame/y"), YPos );
	}
	wxRect restore( const wxRect& defaultDimensions ) const
	{
		wxRect rect;
		rect.height = m_pConfig->Read( wxT("/MainFrame/h"), defaultDimensions.height );
		rect.width = m_pConfig->Read( wxT("/MainFrame/w"), defaultDimensions.width );
		rect.x = m_pConfig->Read( wxT("/MainFrame/x"), defaultDimensions.x );
		rect.y = m_pConfig->Read( wxT("/MainFrame/y"), defaultDimensions.y );
		int displayWidth = 0;
		int displayHeight = 0;
		wxDisplaySize( &displayWidth, &displayHeight );
		if( ( rect.x >= displayWidth ) || ( rect.x < 0 ) )
		{
			rect.x = 0;
		}
		if( ( rect.y >= displayHeight ) || ( rect.y < 0 ) )
		{
			rect.y = 0;
		}
		return rect;
	}
};

//=============================================================================
//================= Implementation ImageAnalysisPlotControls ==================
//=============================================================================
//-----------------------------------------------------------------------------
void ImageAnalysisPlotControls::Load( wxConfigBase* pConfig )
//-----------------------------------------------------------------------------
{
	bool boEnable = pConfig->Read( wxString::Format(wxT("/Controls/%s/Enable"), m_pPlotCanvas->GetConfigName().c_str()), 0l ) != 0;
	m_pCBEnablePlot->SetValue( boEnable );
	m_pPlotCanvas->SetActive( boEnable );
	boEnable = pConfig->Read( wxString::Format(wxT("/Controls/%s/AOIFullMode"), m_pPlotCanvas->GetConfigName().c_str()), 0l ) != 0;
	m_pCBAOIFullMode->SetValue( boEnable );
	m_pPlotCanvas->SetAOIFullMode( boEnable );
	m_pSCAOIh->SetValue( pConfig->Read( wxString::Format(wxT("/Controls/%s/AOIh"), m_pPlotCanvas->GetConfigName().c_str()), 480l ) );
	m_pSCAOIw->SetValue( pConfig->Read( wxString::Format(wxT("/Controls/%s/AOIw"), m_pPlotCanvas->GetConfigName().c_str()), 640l ) );
	m_pSCAOIx->SetValue( pConfig->Read( wxString::Format(wxT("/Controls/%s/AOIx"), m_pPlotCanvas->GetConfigName().c_str()), 0l ) );
	m_pSCAOIy->SetValue( pConfig->Read( wxString::Format(wxT("/Controls/%s/AOIy"), m_pPlotCanvas->GetConfigName().c_str()), 0l ) );
	if( m_pSCDrawStart_percent )
	{
		m_pSCDrawStart_percent->SetValue( pConfig->Read( wxString::Format(wxT("/Controls/%s/DrawStartPercent"), m_pPlotCanvas->GetConfigName().c_str()), 0l ) );
	}
	if( m_pSCDrawWindow_percent )
	{
		m_pSCDrawWindow_percent->SetValue( pConfig->Read( wxString::Format(wxT("/Controls/%s/DrawWindowPercent"), m_pPlotCanvas->GetConfigName().c_str()), 100l ) );
	}
	m_pSCUpdateSpeed->SetValue( pConfig->Read( wxString::Format(wxT("/Controls/%s/UpdateSpeed"), m_pPlotCanvas->GetConfigName().c_str()), 3l ) );
	UpdateControls();
}

//-----------------------------------------------------------------------------
void ImageAnalysisPlotControls::Save( wxConfigBase* pConfig )
//-----------------------------------------------------------------------------
{
	pConfig->Write( wxString::Format(wxT("/Controls/%s/Enable"), m_pPlotCanvas->GetConfigName().c_str()), m_pCBEnablePlot->GetValue() );
	pConfig->Write( wxString::Format(wxT("/Controls/%s/AOIFullMode"), m_pPlotCanvas->GetConfigName().c_str()), m_pCBAOIFullMode->GetValue() );
	pConfig->Write( wxString::Format(wxT("/Controls/%s/AOIh"), m_pPlotCanvas->GetConfigName().c_str()), m_pSCAOIh->GetValue() );
	pConfig->Write( wxString::Format(wxT("/Controls/%s/AOIw"), m_pPlotCanvas->GetConfigName().c_str()), m_pSCAOIw->GetValue() );
	pConfig->Write( wxString::Format(wxT("/Controls/%s/AOIx"), m_pPlotCanvas->GetConfigName().c_str()), m_pSCAOIx->GetValue() );
	pConfig->Write( wxString::Format(wxT("/Controls/%s/AOIy"), m_pPlotCanvas->GetConfigName().c_str()), m_pSCAOIy->GetValue() );
	if( m_pSCDrawStart_percent )
	{
		pConfig->Write( wxString::Format(wxT("/Controls/%s/DrawStartPercent"), m_pPlotCanvas->GetConfigName().c_str()), m_pSCDrawStart_percent->GetValue() );
	}
	if( m_pSCDrawWindow_percent )
	{
		pConfig->Write( wxString::Format(wxT("/Controls/%s/DrawWindowPercent"), m_pPlotCanvas->GetConfigName().c_str()), m_pSCDrawWindow_percent->GetValue() );
	}
	pConfig->Write( wxString::Format(wxT("/Controls/%s/UpdateSpeed"), m_pPlotCanvas->GetConfigName().c_str()), m_pSCUpdateSpeed->GetValue() );
}

//-----------------------------------------------------------------------------
void ImageAnalysisPlotControls::UpdateControls( void )
//-----------------------------------------------------------------------------
{
	bool boActive = m_pCBEnablePlot->IsChecked();
	m_pCBAOIFullMode->Enable( boActive );
	bool boFullAOIMode = m_pCBAOIFullMode->IsChecked();
	m_pSCAOIx->Enable( boActive && !boFullAOIMode );
	m_pSCAOIy->Enable( boActive && !boFullAOIMode );
	m_pSCAOIw->Enable( boActive && !boFullAOIMode );
	m_pSCAOIh->Enable( boActive && !boFullAOIMode );
	if( m_pSCDrawStart_percent )
	{
		m_pSCDrawStart_percent->Enable( boActive );
	}
	if( m_pSCDrawWindow_percent )
	{
		m_pSCDrawWindow_percent->Enable( boActive );
	}
	m_pSCUpdateSpeed->Enable( boActive );
}

//=============================================================================
//================= Implementation PropViewFrame ==============================
//=============================================================================
// ----------------------------------------------------------------------------
// event tables and other macros for wxWidgets
// ----------------------------------------------------------------------------

// the event tables connect the wxWidgets events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(PropViewFrame, wxFrame)
	EVT_MENU(miHelp_About, PropViewFrame::OnAbout)
	EVT_MENU(miQuit, PropViewFrame::OnQuit)
	EVT_MENU(miAction_UseDevice, PropViewFrame::OnUseDevice)
	EVT_MENU(miActionSave_ToDefault, PropViewFrame::OnSaveToDefault)
	EVT_MENU(miActionSave_SystemToDefault, PropViewFrame::OnSaveSystemToDefault)
	EVT_MENU(miActionSave_ActiveDevice, PropViewFrame::OnSaveActiveDevice)
	EVT_MENU(miActionSave_ExportActiveDevice, PropViewFrame::OnExportActiveDevice)
	EVT_MENU(miActionLoad_FromDefault, PropViewFrame::OnLoadFromDefault)
	EVT_MENU(miActionLoad_ActiveDevice, PropViewFrame::OnLoadActiveDevice)
	EVT_MENU(miActionLoad_ImportActiveDevice, PropViewFrame::OnImportActiveDevice)
	EVT_MENU(miAction_SaveImage, PropViewFrame::OnSaveImage)
	EVT_MENU(miAction_SaveImageSequence, PropViewFrame::OnSaveImageSequence)
	EVT_MENU(miAction_UpdateDeviceList, PropViewFrame::OnUpdateDeviceList)
	EVT_MENU(miPopUpPropRestoreDefault, PropViewFrame::OnPopUpPropRestoreDefault)
	EVT_MENU(miPopUpPropAppendValue, PropViewFrame::OnPopUpPropAppendValue)
	EVT_MENU(miPopUpPropDeleteValue, PropViewFrame::OnPopUpPropDeleteValue)
	EVT_MENU(miPopUpMethExec, PropViewFrame::OnPopUpMethExec)
	EVT_MENU(miCapture_Live, PropViewFrame::OnBtnLive)
	EVT_MENU(miCapture_Snap, PropViewFrame::OnBtnSnap)
	EVT_MENU(miCapture_Abort, PropViewFrame::OnBtnAbort)
	EVT_MENU(miCapture_Record, PropViewFrame::OnBtnRecord)
	EVT_MENU(miCapture_Forward, PropViewFrame::OnBtnForward)
	EVT_MENU(miCapture_Backward, PropViewFrame::OnBtnBackward)
	EVT_MENU(miSettingsDisplay_Active, PropViewFrame::OnActivateDisplay)
	EVT_MENU(miSettingsDisplay_FitToScreen, PropViewFrame::OnFitToDisplay)
	EVT_MENU(miSettingsDisplay_OverlayMessages, PropViewFrame::OnShowDisplayMessagesAsOverlay)
	EVT_MENU(miSettingsDisplay_RequestInfoAsOverlay, PropViewFrame::OnShowRequestInfosAsOverlay)
	EVT_MENU(miSettings_ShowConnectedDevicesOnly, PropViewFrame::OnConnectedOnly)
	EVT_MENU(miSettings_ShowLogWindow, PropViewFrame::OnSettingsDisplayLogWindow)
	EVT_MENU(miSettingsPropGrid_Show, PropViewFrame::OnSettingsPropGrid_Show)
	EVT_MENU(miSettingsPropGrid_StandardView, PropViewFrame::OnSettingsPropGrid_StandardView)
	EVT_MENU(miSettingsPropGrid_DevelopersView, PropViewFrame::OnSettingsPropGrid_DevelopersView)
	EVT_SIZE(PropViewFrame::OnSize)
	EVT_TIMER(wxID_ANY, PropViewFrame::OnTimer)
	EVT_BUTTON(widBtnAbort, PropViewFrame::OnBtnAbort)
	EVT_BUTTON(widBtnLive, PropViewFrame::OnBtnLive)
	EVT_BUTTON(widBtnUseDevice, PropViewFrame::OnUseDevice)
	EVT_BUTTON(widBtnSnap, PropViewFrame::OnBtnSnap)
	EVT_COMMAND(widMainFrame, imageReadyEvent, PropViewFrame::OnImageReady)
	EVT_COMMAND(widMainFrame, sequenceReadyEvent, PropViewFrame::OnSequenceReady)
	EVT_COMMAND(widMainFrame, imageSkippedEvent, PropViewFrame::OnImageSkipped)
	EVT_COMMAND(widMainFrame, toggleDisplayArea, PropViewFrame::OnToggleDisplayWindowSize)
	EVT_COMMAND(widMainFrame, liveModeAborted, PropViewFrame::OnLiveModeAborted)
	EVT_COMMAND(widMainFrame, eventNotified, PropViewFrame::OnEventNotified)
	EVT_COMMAND(widMainFrame, imageInfoEvent, PropViewFrame::OnImageInfo)
	EVT_PG_CHANGED(widPropGrid, PropViewFrame::OnPropertyChanged)
	EVT_PG_RIGHT_CLICK(widPropGrid, PropViewFrame::OnPropertyRightClicked)
	EVT_TEXT(widDevCombo, PropViewFrame::OnDevComboTextChanged)
	EVT_TEXT(widInfoPlot_PlotSelectionCombo, PropViewFrame::OnInfoPlot_PlotSelectionComboTextChanged)
	EVT_CLOSE(PropViewFrame::OnClose)
	EVT_SPLITTER_SASH_POS_CHANGED(widHorSplitter, PropViewFrame::OnSplitterSashPosChanged)
	EVT_SPLITTER_SASH_POS_CHANGED(widVerSplitter, PropViewFrame::OnSplitterSashPosChanged)
	EVT_CHECKBOX(widPixelHistogram | iapidCBEnable, PropViewFrame::OnCBEnableAnalysisPlot)
	EVT_CHECKBOX(widNoiseHistogram | iapidCBEnable, PropViewFrame::OnCBEnableAnalysisPlot)
	EVT_CHECKBOX(widLineProfile | iapidCBEnable, PropViewFrame::OnCBEnableAnalysisPlot)
	EVT_CHECKBOX(widPixelHistogram | iapidCBAOIFullMode, PropViewFrame::OnAnalysisPlotCBAOIFullMode)
	EVT_CHECKBOX(widNoiseHistogram | iapidCBAOIFullMode, PropViewFrame::OnAnalysisPlotCBAOIFullMode)
	EVT_CHECKBOX(widLineProfile | iapidCBAOIFullMode, PropViewFrame::OnAnalysisPlotCBAOIFullMode)
	EVT_CHECKBOX(widCBEnableInfoPlot, PropViewFrame::OnCBEnableInfoPlot)
	EVT_TEXT(widPixelHistogram | iapidSCAOIx, PropViewFrame::OnAnalysisPlotAOIxTextChanged)
	EVT_TEXT(widNoiseHistogram | iapidSCAOIx, PropViewFrame::OnAnalysisPlotAOIxTextChanged)
	EVT_TEXT(widLineProfile | iapidSCAOIx, PropViewFrame::OnAnalysisPlotAOIxTextChanged)
	EVT_TEXT(widPixelHistogram | iapidSCAOIy, PropViewFrame::OnAnalysisPlotAOIyTextChanged)
	EVT_TEXT(widNoiseHistogram | iapidSCAOIy, PropViewFrame::OnAnalysisPlotAOIyTextChanged)
	EVT_TEXT(widLineProfile | iapidSCAOIy, PropViewFrame::OnAnalysisPlotAOIyTextChanged)
	EVT_TEXT(widPixelHistogram | iapidSCAOIw, PropViewFrame::OnAnalysisPlotAOIwTextChanged)
	EVT_TEXT(widNoiseHistogram | iapidSCAOIw, PropViewFrame::OnAnalysisPlotAOIwTextChanged)
	EVT_TEXT(widLineProfile | iapidSCAOIw, PropViewFrame::OnAnalysisPlotAOIwTextChanged)
	EVT_TEXT(widPixelHistogram | iapidSCAOIh, PropViewFrame::OnAnalysisPlotAOIhTextChanged)
	EVT_TEXT(widNoiseHistogram | iapidSCAOIh, PropViewFrame::OnAnalysisPlotAOIhTextChanged)
	EVT_TEXT(widLineProfile | iapidSCAOIh, PropViewFrame::OnAnalysisPlotAOIhTextChanged)
	EVT_TEXT(widPixelHistogram | iapidSCDrawStart_percent, PropViewFrame::OnAnalysisPlotDrawStartChanged)
	EVT_TEXT(widNoiseHistogram | iapidSCDrawStart_percent, PropViewFrame::OnAnalysisPlotDrawStartChanged)
	EVT_TEXT(widPixelHistogram | iapidSCDrawWindow_percent, PropViewFrame::OnAnalysisPlotDrawWindowChanged)
	EVT_TEXT(widNoiseHistogram | iapidSCDrawWindow_percent, PropViewFrame::OnAnalysisPlotDrawWindowChanged)
	EVT_TEXT(widPixelHistogram | iapidSCUpdateSpeed, PropViewFrame::OnAnalysisPlotUpdateSpeedTextChanged)
	EVT_TEXT(widNoiseHistogram | iapidSCUpdateSpeed, PropViewFrame::OnAnalysisPlotUpdateSpeedTextChanged)
	EVT_TEXT(widLineProfile | iapidSCUpdateSpeed, PropViewFrame::OnAnalysisPlotUpdateSpeedTextChanged)
	EVT_TEXT(widSCInfoPlotHistoryDepth, PropViewFrame::OnInfoPlotUpdateHistoryDepth)
	EVT_TEXT(widSCInfoPlotUpdateSpeed, PropViewFrame::OnInfoPlotUpdateSpeedTextChanged)
END_EVENT_TABLE()

const wxString PropViewFrame::m_UNKNOWN_STRING(wxT("unknown"));

//-----------------------------------------------------------------------------
PropViewFrame::PropViewFrame( const wxString& title, const wxPoint& pos, const wxSize& size, const wxString& cmdLine )
: wxFrame(0, widMainFrame, title, pos, size), m_boDisplayWindowMaximized(false), m_CurrentImage(1),
   m_CurrentPixelFormat(m_UNKNOWN_STRING), m_CurrentRequestResult(), m_DeviceListChangedCounter(numeric_limits<unsigned int>::max()),
   m_NoDevStr(wxT("no device")), m_pDevPropHandler(0), m_pDisplayArea(0), m_pLogWindow(0), m_pPropGrid(0)
//-----------------------------------------------------------------------------
{
	memset( &m_CurrentRequestInfo, 0, sizeof(m_CurrentRequestInfo) );

	// create the menu
	// sub menu 'Action -> Save'
	wxMenu* pMenuActionSave = new wxMenu;
	m_pMIAction_Save_ToDefault = pMenuActionSave->Append( miActionSave_ToDefault, wxT("to default") );
	m_pMIAction_Save_SystemToDefault = pMenuActionSave->Append( miActionSave_SystemToDefault, wxT("system settings to default") );
	m_pMIAction_Save_ActiveDevice = pMenuActionSave->Append( miActionSave_ActiveDevice, wxT("Active device") );
	m_pMIAction_Save_ExportActiveDevice = pMenuActionSave->Append( miActionSave_ExportActiveDevice, wxT("Export active device"), wxT("Stores the current settings for this device in a platform independent XML file") );

	// sub menu 'Action -> Load'
	wxMenu* pMenuActionLoad = new wxMenu;
	m_pMIAction_Load_FromDefault = pMenuActionLoad->Append( miActionLoad_FromDefault, wxT("from default") );
	m_pMIAction_Load_ActiveDevice = pMenuActionLoad->Append( miActionLoad_ActiveDevice, wxT("Active device") );
	m_pMIAction_Load_ImportActiveDevice = pMenuActionLoad->Append( miActionLoad_ImportActiveDevice, wxT("Import active device"), wxT("Tries to loads settings for this device from a platform independent XML file") );

	// menu 'Action'
	wxMenu* pMenuAction = new wxMenu;
	m_pMIAction_Use = pMenuAction->Append( miAction_UseDevice, wxT("&Use device\tCTRL+U"), wxT("Opens or closes the selected device"), wxITEM_CHECK );
	pMenuAction->Append( wxID_ANY, wxT("Save..."), pMenuActionSave );
	pMenuAction->Append( wxID_ANY, wxT("Load..."), pMenuActionLoad );
	pMenuAction->AppendSeparator();
	m_pMIAction_SaveImage = pMenuAction->Append( miAction_SaveImage, wxT("Save image...") );
	m_pMIAction_SaveImageSequence = pMenuAction->Append( miAction_SaveImageSequence, wxT("Save all recorded images...") );
	pMenuAction->AppendSeparator();
	m_pMIAction_UpdateDeviceList = pMenuAction->Append( miAction_UpdateDeviceList, wxT("&Update device list\tALT+CTRL+U") );
	pMenuAction->AppendSeparator();
	pMenuAction->Append( miQuit, wxT("E&xit\tALT-X") );

	// menu 'Capture'
	wxMenu* pMenuCapture = new wxMenu;
	m_pMICapture_Live = pMenuCapture->Append( miCapture_Live, wxT("&Live\tCTRL+L"), wxT("Starts or stops a continuous acquisition"), wxITEM_CHECK );
	m_pMICapture_Snap = pMenuCapture->Append( miCapture_Snap, wxT("&Snap\tCTRL+S"), wxT("Captures a single image from the selected device") );
	m_pMICapture_Abort = pMenuCapture->Append( miCapture_Abort, wxT("&Abort\tCTRL+A"), wxT("Clears the request queue, but does NOT stop a running continuous acquisition") );
	m_pMICapture_Record = pMenuCapture->Append( miCapture_Record, wxT("&Record\tCTRL+R"), wxT("Records the next 'RequestCount'(System Settings) images."), wxITEM_CHECK  );
	m_pMICapture_Forward = pMenuCapture->Append( miCapture_Forward, wxT("&Forward\tCTRL+F"), wxT("Displays the next recorded image.") );
	m_pMICapture_Backward = pMenuCapture->Append( miCapture_Backward, wxT("&Backward\tCTRL+B"), wxT("Displays the previous recorded image.") );

	// sub menu 'Settings -> Display'
	wxMenu* pMenuSettingsDisplay = new wxMenu;
	m_pMISettings_Display_Active = pMenuSettingsDisplay->Append( miSettingsDisplay_Active, wxT("&Blit active\tALT+CTRL+B"), wxT("If not active images will be captured, but NOT displayed"), wxITEM_CHECK );
	m_pMISettings_Display_FitToScreen = pMenuSettingsDisplay->Append( miSettingsDisplay_FitToScreen, wxT("&Fit to screen\tALT+CTRL+F"), wxT("If active the captured image will be scaled to fit into the display screen(slower)"), wxITEM_CHECK );
	m_pMISettings_Display_ShowMessagesAsOverlay = pMenuSettingsDisplay->Append( miSettingsDisplay_OverlayMessages, wxT("&Message overlay\tALT+CTRL+M"), wxT("If active the various informations will be blitted on top of the image to inform e.g. about possible performance losses"), wxITEM_CHECK );
	m_pMISettings_Display_ShowRequestInfoAsOverlay = pMenuSettingsDisplay->Append( miSettingsDisplay_RequestInfoAsOverlay, wxT("&Request info overlay\tALT+CTRL+R"), wxT("If active the various informations returned together with the image will be displayed as an overlay(This will cost some additional CPU time)"), wxITEM_CHECK );

	// sub menu 'Settings -> Property grid'
	wxMenu* pMenuSettingsPropertyGrid = new wxMenu;
	m_pMISettings_PropGrid_Show = pMenuSettingsPropertyGrid->Append( miSettingsPropGrid_Show, wxT("Show &property grid\tALT+CTRL+P"), wxT(""), wxITEM_CHECK );
	pMenuSettingsPropertyGrid->AppendSeparator();
	m_pMISettings_PropGrid_StandardView = pMenuSettingsPropertyGrid->Append( miSettingsPropGrid_StandardView, wxT("Standard view"), wxT(""), wxITEM_RADIO );
	m_pMISettings_PropGrid_DevelopersView = pMenuSettingsPropertyGrid->Append( miSettingsPropGrid_DevelopersView, wxT("Developers view"), wxT(""), wxITEM_RADIO );

	// menu 'Settings'
	wxMenu* pMenuSettings = new wxMenu;
	pMenuSettings->Append( wxID_ANY, wxT("&Image display..."), pMenuSettingsDisplay );
	pMenuSettings->AppendSeparator();
	m_pMISettings_ShowConnectedOnly = pMenuSettings->Append( miSettings_ShowConnectedDevicesOnly, wxT("Display &connected devices only\tALT+CTRL+C"), wxT(""), wxITEM_CHECK );
	pMenuSettings->AppendSeparator();
	m_pMISettings_Display_LogWindow = pMenuSettings->Append( miSettings_ShowLogWindow, wxT("Show &analysis tabs\tALT+CTRL+A"), wxT(""), wxITEM_CHECK );
	pMenuSettings->Append( wxID_ANY, wxT("Property grid..."), pMenuSettingsPropertyGrid );

	// menu 'About'
	wxMenu* pMenuAbout = new wxMenu;
	pMenuAbout->Append( miHelp_About, wxT("&About"));

	// add all menus to the menu bar
	wxMenuBar* pMenuBar = new wxMenuBar;
	pMenuBar->Append( pMenuAction, wxT("&Action") );
	pMenuBar->Append( pMenuCapture, wxT("&Capture") );
	pMenuBar->Append( pMenuSettings, wxT("&Settings") );
	pMenuBar->Append( pMenuAbout, wxT("&Help") );
	// ... and attach this menu bar to the frame
	SetMenuBar(pMenuBar);

	// define the applications icon
	wxIcon icon(mvIcon_xpm);
	SetIcon( icon );

	// create all GUI elements
	/*

			|-------------------|
			|buttons            |
			|-------------------|
			|prop-  | display   |
			|grid   |           |
			|       |           |
			|       |-----------|
			|       | output    |
			|       |           |
			|-------------------|
	*/
	wxBoxSizer* pTopDownSizer = new wxBoxSizer( wxVERTICAL );
	wxPanel* pPanel = new wxPanel( this );

	m_pToolBar = CreateToolBar( wxNO_BORDER | wxHORIZONTAL | wxTB_FLAT | wxTB_TEXT );
	m_pToolBar->SetMargins( 5, 5 );
	m_pToolBar->SetToolBitmapSize( wxSize(32, 16) );
	m_pToolBar->SetToolSeparation( 10 );

	wxStaticText *pStaticText = new wxStaticText(m_pToolBar, -1, wxT("  Serial No  "));
	m_pToolBar->AddControl( pStaticText );

	m_pDevCombo = new wxComboBox( m_pToolBar, widDevCombo, wxT(" Serial "), wxDefaultPosition, wxSize(120,wxDefaultCoord), 0, NULL, wxCB_DROPDOWN | wxCB_READONLY );
	m_pDevCombo->Select(0);
	m_pDevCombo->SetToolTip( wxT("Contains a list of serial numbers of every device detected. If devices seem to miss make sure these are connected AND a driver for the corresponding device family has been installed") );
	m_pToolBar->AddControl( m_pDevCombo );

	pStaticText = new wxStaticText(m_pToolBar, -1, wxT("  "));
	m_pToolBar->AddControl( pStaticText );
	m_pToolBar->AddTool( miAction_UseDevice, wxT("Use"), Use_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Opens or closes the selected device"),
		wxT("Opens or closes the selected device") );
	m_pToolBar->AddSeparator();
	m_pToolBar->AddTool( miCapture_Live, wxT("Live"), live_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Starts or stops continuous acquisition from the selected device"),
		wxT("Starts or stops continuous acquisition from the selected device") );
	m_pToolBar->AddTool( miCapture_Snap, wxT("Snap"), photo_xpm, wxNullBitmap, wxITEM_NORMAL,
		wxT("Captures a single image from the selected device"),
		wxT("Captures a single image from the selected device") );
	m_pToolBar->AddTool( miCapture_Abort, wxT("Abort"), abort_xpm, wxNullBitmap, wxITEM_NORMAL,
		wxT("Empties the image request queue for the selected device but does NOT stop a running continuous acquisition"),
		wxT("Empties the image request queue for the selected device but does NOT stop a running continuous acquisition") );
	m_pToolBar->AddSeparator();
	m_pToolBar->AddTool( miCapture_Record, wxT("Rec."), record_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Records the next 'RequestCount'(System Settings) images."),
		wxT("Records the next 'RequestCount'(System Settings) images.") );
	m_pToolBar->AddTool( miCapture_Backward, wxT("Prev."), back_xpm, wxNullBitmap, wxITEM_NORMAL,
		wxT("Displays the previous recorded image"),
		wxT("Displays the previous recorded image") );
	m_pToolBar->AddTool( miCapture_Forward, wxT("Next"), forward_xpm, wxNullBitmap, wxITEM_NORMAL,
		wxT("Displays the next recorded image"),
		wxT("Displays the next recorded image") );
	m_pToolBar->AddSeparator();
	m_pToolBar->AddTool( miSettingsDisplay_Active, wxT("Blit"), DisplayActive_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Turns the display on or off"),
		wxT("Turns the display on or off") );
	m_pToolBar->AddTool( miSettingsDisplay_FitToScreen, wxT("Fit"), Scale_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("If active the captured image will be scaled to fit into the display area(needs additional CPU time)"),
		wxT("If active the captured image will be scaled to fit into the display area(needs additional CPU time)") );
	m_pToolBar->AddTool( miSettingsDisplay_OverlayMessages, wxT("Msg."), DisplayMessages_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Turns display messages on or off"),
		wxT("Turns display messages on or off") );
	m_pToolBar->AddTool( miSettingsDisplay_RequestInfoAsOverlay, wxT("Info"), DisplayMessages_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Turns request info overlay messages on or off"),
		wxT("If active the various informations returned together with the image will be displayed as an overlay(This will cost some additional CPU time)") );
	m_pToolBar->AddSeparator();
	m_pToolBar->AddTool( miSettings_ShowLogWindow, wxT("Analysis"), LogWindow_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Displays or hides the analysis tabs"),
		wxT("Displays or hides the analysis tabs") );
	m_pToolBar->AddTool( miSettingsPropGrid_Show, wxT("Grid"), PropertyGrid_xpm, wxNullBitmap, wxITEM_CHECK,
		wxT("Displays or hides the device property grid"),
		wxT("Displays or hides the device property grid") );
	m_pToolBar->AddSeparator();
	m_pToolBar->AddTool( miHelp_About, wxT("About"), help_xpm, wxNullBitmap, wxITEM_NORMAL,
		wxT("Displays the about dialog"),
		wxT("Displays the about dialog") );
	m_pToolBar->Realize();

	// splitter for property grid and the windows on the right
	m_pVerticalSplitter = new wxSplitterWindow( pPanel, widVerSplitter, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER );

	// property grid on the left
	m_pPropGrid = new wxPropertyGrid( m_pVerticalSplitter, widPropGrid, wxDefaultPosition, wxDefaultSize,
		//wxPG_BOLD_MODIFIED |
		wxPG_SPLITTER_AUTO_CENTER |
		//wxPG_HIDE_MARGIN |
		//wxPG_STATIC_SPLITTER |
		wxPG_TOOLTIPS |
		wxTAB_TRAVERSAL |
		wxSUNKEN_BORDER
		);
	m_pPropGrid->SetExtraStyle( wxPG_EX_MODE_BUTTONS );
	m_pPropGrid->SetExtraStyle( wxPG_EX_HELP_AS_TOOLTIPS );
	//m_pPropGrid->SetExtraStyle( wxPG_EX_GREY_LABEL_WHEN_DISABLED ); // this will display lists as disabled as well...
#if wxPROPGRID_MINOR < 3
	m_pPropGrid->Compact(true);
#endif
	m_pPropGrid->SetLineColour( wxColour( 128, 128, 128 ) );

	// splitter between the image display window and the notebook window in the lower right corner of the window
	m_pHorizontalSplitter = new wxSplitterWindow( m_pVerticalSplitter, widHorSplitter, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER );

	// the window for displaying the images
	m_pDisplayArea = new ImageCanvas( this, m_pHorizontalSplitter, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN | wxHSCROLL | wxVSCROLL );
	m_pDisplayArea->EnableScrolling( true, true );

	// the window for the controls in the lower right corner of the image
	m_pLowerRightWindow = new wxNotebook( m_pHorizontalSplitter, wxID_ANY, wxDefaultPosition, wxDefaultSize );

	// log window (page of lower right notebook)
	m_pLogWindow = new wxTextCtrl( m_pLowerRightWindow, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxSUNKEN_BORDER | wxTE_RICH | wxTE_READONLY );
	m_pLowerRightWindow->AddPage( m_pLogWindow, wxT("Output"), true );
	
	wxPanel* pAnalysisPanel = 0;
	pAnalysisPanel = CreateImageAnalysisPlotControls( m_pLowerRightWindow, widPixelHistogram );
	m_pLowerRightWindow->AddPage( pAnalysisPanel, m_ImageAnalysisPlots[GetAnalysisControlIndex( widPixelHistogram )].m_pPlotCanvas->GetName(), false );
	pAnalysisPanel = CreateImageAnalysisPlotControls( m_pLowerRightWindow, widNoiseHistogram );
	m_pLowerRightWindow->AddPage( pAnalysisPanel , m_ImageAnalysisPlots[GetAnalysisControlIndex( widNoiseHistogram )].m_pPlotCanvas->GetName(), false );
	pAnalysisPanel = CreateImageAnalysisPlotControls( m_pLowerRightWindow, widLineProfile );
	m_pLowerRightWindow->AddPage( pAnalysisPanel , m_ImageAnalysisPlots[GetAnalysisControlIndex( widLineProfile )].m_pPlotCanvas->GetName(), false );

	// info plot window (page of lower right notebook)
	wxPanel* pPanelInfoPlot = new wxPanel( m_pLowerRightWindow );
	wxBoxSizer* pInfoPlotPanelSizer = new wxBoxSizer( wxHORIZONTAL );
	wxBoxSizer* pInfoPlotControlsSizer = new wxBoxSizer( wxVERTICAL );
	m_pCBEnableInfoPlot = new wxCheckBox( pPanelInfoPlot, widCBEnableInfoPlot, wxT("Enable") );
	pInfoPlotControlsSizer->Add( m_pCBEnableInfoPlot, wxSizerFlags().Left() );
	pInfoPlotControlsSizer->AddSpacer( 5 );

	wxFlexGridSizer* pInfoPlotControlsGridSizer = new wxFlexGridSizer( 2, 3 );
	pInfoPlotControlsGridSizer->Add( new wxStaticText( pPanelInfoPlot, wxID_ANY, wxT("update interval:") ) );
	m_pSCInfoPlotUpdateSpeed = new wxSpinCtrl( pPanelInfoPlot, widSCInfoPlotUpdateSpeed, wxT("3"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, PlotCanvas::ulMin, PlotCanvas::ulMax, 3 );
	pInfoPlotControlsGridSizer->Add( m_pSCInfoPlotUpdateSpeed, wxSizerFlags().Left() );
	pInfoPlotControlsGridSizer->Add( new wxStaticText( pPanelInfoPlot, wxID_ANY, wxT("History depth:") ) );
	m_pSCInfoPlotHistoryDepth = new wxSpinCtrl( pPanelInfoPlot, widSCInfoPlotHistoryDepth, wxT("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 5, 10000, 20 );
	pInfoPlotControlsGridSizer->Add( m_pSCInfoPlotHistoryDepth, wxSizerFlags().Left() );
	pInfoPlotControlsGridSizer->Add( new wxStaticText( pPanelInfoPlot, wxID_ANY, wxT("Plot selection:") ) );
	m_pInfoPlotSelectionCombo = new wxComboBox( pPanelInfoPlot, widInfoPlot_PlotSelectionCombo, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_DROPDOWN );
	pInfoPlotControlsGridSizer->Add( m_pInfoPlotSelectionCombo, wxSizerFlags().Left() );
	m_pInfoPlotSelectionCombo->Select(0);
	m_pInfoPlotSelectionCombo->SetToolTip( wxT("Contains a list of possible polts that can be displayed on this page") );

	pInfoPlotControlsSizer->Add( pInfoPlotControlsGridSizer );
	pInfoPlotPanelSizer->Add( pInfoPlotControlsSizer, wxSizerFlags().Left() );
	m_pInfoPlotArea = new InfoPlotCanvas( pPanelInfoPlot, wxID_ANY, wxDefaultPosition, wxDefaultSize );
	for( unsigned int i=0; i<InfoPlotCanvas::psLAST; i++ )
	{
		int item = m_pInfoPlotSelectionCombo->Append( m_pInfoPlotArea->GetPlotSelectionName( static_cast<InfoPlotCanvas::TPlotSelection>(i) ) );
		m_pInfoPlotSelectionCombo->SetClientData( item, reinterpret_cast<void*>(i) );
	}
	pInfoPlotPanelSizer->Add( m_pInfoPlotArea, wxSizerFlags(1).Expand() );
	pPanelInfoPlot->SetSizer( pInfoPlotPanelSizer );
	m_pLowerRightWindow->AddPage( pPanelInfoPlot, wxT("Info plot"), false );

	// splitter
	m_pHorizontalSplitter->SetMinimumPaneSize( 175 );
	m_pHorizontalSplitter->SetSashGravity( 0.9 );
	m_pVerticalSplitter->SetMinimumPaneSize( 50 );
	pTopDownSizer->Add( m_pVerticalSplitter, wxSizerFlags(6).Expand() );

	m_pStatusBar = new wxStatusBar( pPanel, widStatusBar );
	pTopDownSizer->Add( m_pStatusBar, wxSizerFlags().Expand() );
	pPanel->SetSizer( pTopDownSizer );
	pTopDownSizer->SetSizeHints( this );
	
	// restore previous state
	FramePositionStorage frameStorage(this);
	wxRect defaultRect(0, 0, 1024, 700);
	wxRect rect = frameStorage.restore( defaultRect );
	wxConfigBase *pConfig = wxConfigBase::Get();
	m_VerticalSplitterPos = pConfig->Read( wxT("/MainFrame/verticalSplitter"), -1l );
	m_HorizontalSplitterPos = pConfig->Read( wxT("/MainFrame/horizontalSplitter"), -1l );

	bool boActive = pConfig->Read( wxT("/MainFrame/Settings/scaleDisplay"), 0l ) != 0;
	m_pMISettings_Display_FitToScreen->Check( boActive );
	m_pToolBar->ToggleTool( miSettingsDisplay_FitToScreen, boActive );
	m_pDisplayArea->SetScaling( boActive );
	boActive = pConfig->Read( wxT("/MainFrame/Settings/overlayMessages"), 0l ) != 0;
	m_pMISettings_Display_ShowMessagesAsOverlay->Check( boActive );
	m_pDisplayArea->SetPerformanceWarningOutput( boActive );
	m_pToolBar->ToggleTool( miSettingsDisplay_OverlayMessages, boActive );
	boActive = pConfig->Read( wxT("/MainFrame/Settings/requestInfoOverlay"), 0l ) != 0;
	m_pMISettings_Display_ShowRequestInfoAsOverlay->Check( boActive );
	m_pDisplayArea->SetInfoOverlayMode( boActive );
	m_pToolBar->ToggleTool( miSettingsDisplay_RequestInfoAsOverlay, boActive );
	boActive = pConfig->Read( wxT("/MainFrame/Settings/displayActive"), 1l ) != 0;
	m_pMISettings_Display_Active->Check( boActive );
	m_pToolBar->ToggleTool( miSettingsDisplay_Active, boActive );
	m_pDisplayArea->SetActive( boActive );
	boActive = pConfig->Read( wxT("/MainFrame/Settings/activeDevicesOnly"), 0l ) != 0;
	m_pMISettings_ShowConnectedOnly->Check( boActive );
	boActive = pConfig->Read( wxT("/MainFrame/Settings/displayLogWindow"), 1l ) != 0;
	m_pMISettings_Display_LogWindow->Check( boActive );
	m_pToolBar->ToggleTool( miSettings_ShowLogWindow, boActive );
	if( boActive )
	{
		m_pHorizontalSplitter->SplitHorizontally( m_pDisplayArea, m_pLowerRightWindow, 0 );
	}
	else
	{
		m_pHorizontalSplitter->Initialize( m_pDisplayArea );
	}
	boActive = pConfig->Read( wxT("/MainFrame/Settings/displayPropGrid"), 1l ) != 0;
	m_pMISettings_PropGrid_Show->Check( boActive );
	m_pToolBar->ToggleTool( miSettingsPropGrid_Show, boActive );
	if( boActive )
	{
		m_pVerticalSplitter->SplitVertically( m_pPropGrid, m_pHorizontalSplitter, 0 );
	}
	else
	{
		m_pVerticalSplitter->Initialize( m_pHorizontalSplitter );
	}
	int selectedPage = pConfig->Read( wxT("/MainFrame/selectedPage"), -1l );
	if( selectedPage != -1 )
	{
		m_pLowerRightWindow->SetSelection( selectedPage );
	}

	// previous view mode
	m_ViewMode = static_cast<DevicePropertyHandler::TViewMode>(pConfig->Read( wxT("/MainFrame/Settings/propGridViewMode"), DevicePropertyHandler::vmStandard ));

	// property grid
	double splitterratio;
	pConfig->Read( wxT("/PropGrid/splitterRatio"), &splitterratio, 0.5 );

	for( unsigned int i=0; i<iapLAST; i++ )
	{
		m_ImageAnalysisPlots[i].Load( pConfig );
	}

	// info plot
	bool boInfoPlotActive = pConfig->Read( wxT("/Controls/InfoPlot/Enable"), 0l ) != 0;
	m_pCBEnableInfoPlot->SetValue( boInfoPlotActive );
	m_pInfoPlotArea->SetActive( boInfoPlotActive );
	m_pSCInfoPlotHistoryDepth->SetValue( pConfig->Read( wxT("/Controls/InfoPlot/HistoryDepth"), 20l ) );
	m_pSCInfoPlotUpdateSpeed->SetValue( pConfig->Read( wxT("/Controls/InfoPlot/UpdateSpeed"), 3l ) );
	m_pInfoPlotSelectionCombo->Select( pConfig->Read( wxT("/Controls/InfoPlot/PlotSelection"), 0l ) );
	InfoPlotCanvas::TPlotSelection plotSelection = static_cast<InfoPlotCanvas::TPlotSelection>(wxPtrToUInt( m_pInfoPlotSelectionCombo->GetClientData( m_pInfoPlotSelectionCombo->GetSelection() ) ));
	m_pInfoPlotArea->SetPlotSelection( plotSelection );

	// scan command line
	bool boDisplayDebugInfo = false;
	bool boDisplayFullTree = false;
	bool boDisplayInvisibleComponents = false;
	bool boOpenDeviceInLiveMode = false;
	bool boShowEvents = false;
	wxString processedParameters;
	wxString serialToOpen;
	if( !cmdLine.IsEmpty() )
	{
		wxString param, key, value;
		wxString truncatedCmdLine(cmdLine);
		truncatedCmdLine.Trim( true );
		truncatedCmdLine.Trim( false );
		truncatedCmdLine.append( wxT(" ") );
		// now the string has NO spaces in the beginning an ONE at the end
		while( ( param = truncatedCmdLine.BeforeFirst( wxT(' ') ) ) != wxT("") )
		{
			key = param.BeforeFirst( wxT('=') );
			value = param.AfterFirst( wxT('=') );
			if( key.IsEmpty() || value.IsEmpty() )
			{
				WriteErrorMessage( wxString::Format( wxT("%s: Invalid command line parameter: '%s'.\n"), __FUNCTION__, param.c_str() ) );
			}
			else
			{
				if( ( key == wxT("width") ) || ( key == wxT("w") ) )
				{
					rect.width = atoi( value.mb_str() );
				}
				else if( ( key == wxT("height") ) || ( key == wxT("h") ) )
				{
					rect.height = atoi( value.mb_str() );
				}
				else if( ( key == wxT("xpos") ) || ( key == wxT("x") ) )
				{
					rect.x = atoi( value.mb_str() );
				}
				else if( ( key == wxT("ypos") ) || ( key == wxT("y") ) )
				{
					rect.y = atoi( value.mb_str() );
				}
				else if( key == wxT("splitterratio") )
				{
					splitterratio = atof( value.mb_str() );
				}
				else if( ( key == wxT("propgridwidth") ) || ( key == wxT("pgw") ) )
				{
					m_VerticalSplitterPos = atoi( value.mb_str() );
				}
				else if( ( key == wxT("debuginfo") ) || ( key == wxT("di") ) )
				{
					boDisplayDebugInfo = ( atoi( value.mb_str() ) != 0 );
				}
				else if( ( key == wxT("dic") ) )
				{
					boDisplayInvisibleComponents = ( atoi( value.mb_str() ) != 0 );
				}
				else if( ( key == wxT("fulltree") ) || ( key == wxT("ft") ) )
				{
					boDisplayFullTree = ( atoi( value.mb_str() ) != 0 );
				}
				else if( ( key == wxT("device") ) || ( key == wxT("d") ) )
				{
					serialToOpen = value;
				}
				else if( key == wxT("live") )
				{
					boOpenDeviceInLiveMode = ( atoi( value.mb_str() ) != 0 );
				}
				else if( ( key == wxT("showEvents") ) || ( key == wxT("se") ) )
				{
					boShowEvents = ( atoi( value.mb_str() ) != 0 );
				}
				else
				{
					WriteErrorMessage( wxString::Format( wxT("%s: Invalid command line parameter: '%s'.\n"), __FUNCTION__, param.c_str() ) );
				}
				processedParameters += param;
				processedParameters.Append( wxT(' ') );
				truncatedCmdLine = truncatedCmdLine.AfterFirst( wxT(' ') );
			}
		}
	}

	wxTextAttr boldStyle;
	m_pLogWindow->GetStyle( m_pLogWindow->GetLastPosition(), boldStyle );
	wxFont boldFont(boldStyle.GetFont());
	boldFont.SetWeight( wxFONTWEIGHT_BOLD );
	boldFont.SetPointSize( 10 );
	boldFont.SetUnderlined( true );
	boldStyle.SetFont( boldFont );
	WriteLogMessage( wxT("Available command line options:\n" ), boldStyle );
	WriteLogMessage( wxT("'width'         or 'w'   defines the startup width of the application\n") );
	WriteLogMessage( wxT("'height'        or 'h'   defines the startup height of the application\n") );
	WriteLogMessage( wxT("'xpos'          or 'x'   defines the startup x position of the application\n") );
	WriteLogMessage( wxT("'ypos'          or 'y'   defines the startup y position of the application\n") );
	WriteLogMessage( wxT("'splitterratio'          defines the startup ratio of the position of the property grids splitter\n") );
	WriteLogMessage( wxT("'propgridwidth' or 'pgw' defines the startup width of the property grid\n") );
	WriteLogMessage( wxT("'debuginfo'     or 'di'  to display debug information in the property grid\n") );
	WriteLogMessage( wxT("'dic'                    to display invisible(currently shadowed) components in the property grid\n") );
	WriteLogMessage( wxT("'fulltree'      or 'ft'  to display the complete property tree(including the data not meant to be accessed by the user\n") );
	WriteLogMessage( wxT("'device'        or 'd'   to directly open a device with a particular serial number\n") );
	WriteLogMessage( wxT("'live'                   to start live acquisition from the device openen via 'device' or 'd' directly\n") );
	WriteLogMessage( wxT("'showEvents'    or 'se'  to display occuring events in the log output window (this will cause additional CPU load)\n") );
	WriteLogMessage( wxT("\nUsage example: '<app name> di=1 h=340'\n") );

	WriteLogMessage( wxT("processed command line parameters: "), boldStyle );
	WriteLogMessage( ( processedParameters.length() > 0 ) ? processedParameters : wxT("none"), boldStyle );
	WriteLogMessage( wxT('\n'), boldStyle );

	wxTextAttr style( wxColour(0, 0, 255) );
	WriteLogMessage( wxT("\nRight-click on a method object to execute it.\n"), style );
	WriteLogMessage( wxT("Parameters must be separated by whitespaces, empty strings can be passed as a single underscore.\n"), style );
	WriteLogMessage( wxT("To find out what the parameters please have a look in the manuals interface reference.\n"), style );
	WriteLogMessage( wxT("Right-click on a list or property to restore its default.\n"), style );
	WriteLogMessage( wxT("\n") );
	WriteLogMessage( wxT("Left-click in the display area and press '+' or '-' or use the mouse wheel for zooming in and out of the image(this will only be active when the 'fit to screen mode is switched off).\n"), style );

#ifdef BUILD_WITH_PLUGIN_SUPPORT
	WriteLogMessage( wxString::Format( wxT("Deteced plugins: %d.\n"), m_pluginManager.getCount() ), style );
#endif // #ifdef BUILD_WITH_PLUGIN_SUPPORT

	m_pDevPropHandler = new DevicePropertyHandler( m_pPropGrid, boDisplayDebugInfo, boDisplayFullTree, boDisplayInvisibleComponents, boShowEvents );
	m_pDevPropHandler->SetViewMode( m_ViewMode );
	switch( m_ViewMode )
	{
	case DevicePropertyHandler::vmDeveloper:
		m_pMISettings_PropGrid_DevelopersView->Check();
		break;
	default:
		m_pMISettings_PropGrid_StandardView->Check();
		break;
	}

	SetSizeHints( 720, 400 );
	SetSize( rect );

	m_pHorizontalSplitter->SetSashPosition( ( m_HorizontalSplitterPos != -1 ) ? m_HorizontalSplitterPos : 3*rect.height/4, true );
	m_pVerticalSplitter->SetSashPosition( ( m_VerticalSplitterPos != -1 ) ? m_VerticalSplitterPos : rect.width/3, true );

	SetupDlgControls();
	UpdateTitle();

	if( splitterratio <= 0 || splitterratio >= 1 )
	{
		splitterratio = 0.5;
	}
	m_pPropGrid->SetSplitterRatio( splitterratio );
	UpdatePropGridSplitter();
	wxToolTip::Enable( true );

	m_ListUpdateTimer.SetOwner( this, teListUpdate );
	m_ListUpdateTimer.Start( TIMER_PERIOD );

	if( serialToOpen.length() > 0 )
	{
		int selection = m_pDevCombo->FindString( serialToOpen );
		if( selection != wxNOT_FOUND )
		{
			m_pDevCombo->Select( selection );
			UpdateDeviceFromComboBox();
			ToggleCurrentDevice();
			if( boOpenDeviceInLiveMode )
			{
				ConfigureLive();
			}
		}
		else
		{
			style.SetTextColour( wxColour(255, 0, 0) );
			wxString msg(wxT("\nCan't directly open device "));
			msg.Append(ConvertedString(serialToOpen));
			msg.Append( wxT(" as it doesn't seem to be present or the serial number is incorrect\n"));
			WriteLogMessage( msg, style );
		}
	}
}

//-----------------------------------------------------------------------------
PropViewFrame::~PropViewFrame()
//-----------------------------------------------------------------------------
{
	Deinit();

	// store the current state of the application
	wxConfigBase *pConfig = wxConfigBase::Get();
	// main frame
	FramePositionStorage frameStorage(this);
	frameStorage.store();
	pConfig->Write( wxT("/MainFrame/verticalSplitter"), ( m_pVerticalSplitter->IsSplit() ) ? m_pVerticalSplitter->GetSashPosition() : m_VerticalSplitterPos );
	pConfig->Write( wxT("/MainFrame/horizontalSplitter"), ( m_pHorizontalSplitter->IsSplit() ) ? m_pHorizontalSplitter->GetSashPosition() : m_HorizontalSplitterPos );
	pConfig->Write( wxT("/MainFrame/selectedPage"), m_pLowerRightWindow->GetSelection() );
	pConfig->Write( wxT("/MainFrame/Settings/scaleDisplay"), m_pDisplayArea->IsScaled() );
	pConfig->Write( wxT("/MainFrame/Settings/overlayMessages"), m_pDisplayArea->OverlayMessagesActive() );
	pConfig->Write( wxT("/MainFrame/Settings/requestInfoOverlay"), m_pDisplayArea->InfoOverlayActive() );
	pConfig->Write( wxT("/MainFrame/Settings/displayActive"), m_pDisplayArea->IsActive() );
	pConfig->Write( wxT("/MainFrame/Settings/activeDevicesOnly"), m_pMISettings_ShowConnectedOnly->IsChecked() );
	pConfig->Write( wxT("/MainFrame/Settings/displayLogWindow"), m_pMISettings_Display_LogWindow->IsChecked() );
	pConfig->Write( wxT("/MainFrame/Settings/displayPropGrid"), m_pMISettings_PropGrid_Show->IsChecked() );
	pConfig->Write( wxT("/MainFrame/Settings/propGridViewMode"), int(m_ViewMode) );

	// property grid
	pConfig->Write( wxT("/PropGrid/splitterRatio"), m_pPropGrid->GetSplitterRatio() );

	for( unsigned int i=0; i<iapLAST; i++ )
	{
		m_ImageAnalysisPlots[i].Save( pConfig );
	}

	// info plot
	pConfig->Write( wxT("/Controls/InfoPlot/Enable"), m_pCBEnableInfoPlot->GetValue() );
	pConfig->Write( wxT("/Controls/InfoPlot/HistoryDepth"), m_pSCInfoPlotHistoryDepth->GetValue() );
	pConfig->Write( wxT("/Controls/InfoPlot/UpdateSpeed"), m_pSCInfoPlotUpdateSpeed->GetValue() );
	pConfig->Write( wxT("/Controls/InfoPlot/PlotSelection"), m_pInfoPlotSelectionCombo->GetSelection() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::Abort( void )
//-----------------------------------------------------------------------------
{
	FunctionInterface* p = 0;
	m_pDevPropHandler->GetActiveDevice( &p );
	if( p )
	{
		p->imageRequestReset( 0, 0 );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::ConfigureLive( void )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );

	if( pThread )
	{
		if( !pThread->GetLiveMode() )
		{
			SetupUpdateFrequencies( true );
			pThread->SetLiveMode( true );
		}
		else
		{
			pThread->SetLiveMode( false );
		}
		SetupDlgControls();
	}
}

//-----------------------------------------------------------------------------
wxPanel* PropViewFrame::CreateImageAnalysisPlotControls( wxWindow* pParent, int windowIDOffset )
//-----------------------------------------------------------------------------
{
	int index = GetAnalysisControlIndex( windowIDOffset );
	wxPanel* pPanel = new wxPanel(pParent);
	wxBoxSizer* pSizer = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* pControlsSizer = new wxBoxSizer(wxVERTICAL);
	m_ImageAnalysisPlots[index].m_pCBEnablePlot = new wxCheckBox( pPanel, windowIDOffset + iapidCBEnable, wxT("Enable") );
	pControlsSizer->Add( m_ImageAnalysisPlots[index].m_pCBEnablePlot, wxSizerFlags().Left() );
	pControlsSizer->AddSpacer( 5 );
	m_ImageAnalysisPlots[index].m_pCBAOIFullMode = new wxCheckBox( pPanel, windowIDOffset + iapidCBAOIFullMode, wxT("Full AOI mode") );
	pControlsSizer->Add( m_ImageAnalysisPlots[index].m_pCBAOIFullMode, wxSizerFlags().Left() );
	pControlsSizer->AddSpacer( 5 );
	switch( windowIDOffset )
	{
	case widPixelHistogram:
		m_ImageAnalysisPlots[index].m_pPlotCanvas = new PixelHistogramCanvas( pPanel, wxID_ANY, wxDefaultPosition, wxDefaultSize );
		m_ImageAnalysisPlots[index].m_pPlotCanvas->SetAOIColour( wxColor(128, 0, 255) );
		break;
	case widNoiseHistogram:
		m_ImageAnalysisPlots[index].m_pPlotCanvas = new NoiseHistogramCanvas( pPanel, wxID_ANY, wxDefaultPosition, wxDefaultSize );
		m_ImageAnalysisPlots[index].m_pPlotCanvas->SetAOIColour( wxColor(255, 0, 128) );
		break;
	case widLineProfile:
		m_ImageAnalysisPlots[index].m_pPlotCanvas = new LineProfileCanvas( pPanel, wxID_ANY, wxDefaultPosition, wxDefaultSize );
		break;
	default:
		wxASSERT( false && "Invalid window ID offset for this function" );
		return 0;
	}
	wxFlexGridSizer* pGridSizer = new wxFlexGridSizer(2, m_ImageAnalysisPlots[index].m_pPlotCanvas->HasPercentageWindow() ? 7 : 5);
	pGridSizer->Add( new wxStaticText( pPanel, wxID_ANY, wxT("AOI x-offset:") ) );
	m_ImageAnalysisPlots[index].m_pSCAOIx = new wxSpinCtrl( pPanel, windowIDOffset + iapidSCAOIx, wxT("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 639, 0 );
	pGridSizer->Add( m_ImageAnalysisPlots[index].m_pSCAOIx, wxSizerFlags().Left() );
	pGridSizer->Add( new wxStaticText( pPanel, wxID_ANY, wxT("AOI y-offset:") ) );
	m_ImageAnalysisPlots[index].m_pSCAOIy = new wxSpinCtrl( pPanel, windowIDOffset + iapidSCAOIy, wxT("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 479, 0 );
	pGridSizer->Add( m_ImageAnalysisPlots[index].m_pSCAOIy, wxSizerFlags().Left() );
	pGridSizer->Add( new wxStaticText( pPanel, wxID_ANY, wxT("AOI width:") ) );
	m_ImageAnalysisPlots[index].m_pSCAOIw = new wxSpinCtrl( pPanel, windowIDOffset + iapidSCAOIw, wxT("640"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 640, 640 );
	pGridSizer->Add( m_ImageAnalysisPlots[index].m_pSCAOIw, wxSizerFlags().Left() );
	pGridSizer->Add( new wxStaticText( pPanel, wxID_ANY, wxT("AOI height:") ) );
	m_ImageAnalysisPlots[index].m_pSCAOIh = new wxSpinCtrl( pPanel, windowIDOffset + iapidSCAOIh, wxT("480"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 480, 480 );
	pGridSizer->Add( m_ImageAnalysisPlots[index].m_pSCAOIh, wxSizerFlags().Left() );
	if( m_ImageAnalysisPlots[index].m_pPlotCanvas->HasPercentageWindow() )
	{
		pGridSizer->Add( new wxStaticText( pPanel, wxID_ANY, wxT("Draw start(%):") ) );
		m_ImageAnalysisPlots[index].m_pSCDrawStart_percent = new wxSpinCtrl( pPanel, windowIDOffset + iapidSCDrawStart_percent, wxT("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 99, 100 );
		pGridSizer->Add( m_ImageAnalysisPlots[index].m_pSCDrawStart_percent, wxSizerFlags().Left() );
		pGridSizer->Add( new wxStaticText( pPanel, wxID_ANY, wxT("Draw window(%):") ) );
		m_ImageAnalysisPlots[index].m_pSCDrawWindow_percent = new wxSpinCtrl( pPanel, windowIDOffset + iapidSCDrawWindow_percent, wxT("100"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 100, 100 );
		pGridSizer->Add( m_ImageAnalysisPlots[index].m_pSCDrawWindow_percent, wxSizerFlags().Left() );
	}
	pGridSizer->Add( new wxStaticText( pPanel, wxID_ANY, wxT("update interval:") ) );
	m_ImageAnalysisPlots[index].m_pSCUpdateSpeed = new wxSpinCtrl( pPanel, windowIDOffset + iapidSCUpdateSpeed, wxT("3"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, PlotCanvas::ulMin, PlotCanvas::ulMax, 3 );
	pGridSizer->Add( m_ImageAnalysisPlots[index].m_pSCUpdateSpeed, wxSizerFlags().Left() );
	pControlsSizer->Add( pGridSizer );
	pSizer->Add( pControlsSizer, wxSizerFlags().Left() );
	m_ImageAnalysisPlots[index].m_pPlotCanvas->RegisterImageCanvas( m_pDisplayArea );
	pSizer->Add( m_ImageAnalysisPlots[index].m_pPlotCanvas, wxSizerFlags(1).Expand() );
	pPanel->SetSizer( pSizer );
	return pPanel;
}

//-----------------------------------------------------------------------------
void PropViewFrame::Deinit( void )
//-----------------------------------------------------------------------------
{
	if( m_ListUpdateTimer.IsRunning() )
	{
		m_ListUpdateTimer.Stop();
	}

	delete m_pDevPropHandler;
	m_pDevPropHandler = 0;
	m_pDisplayArea->SetImage(0);
}

//-----------------------------------------------------------------------------
int PropViewFrame::GetAnalysisControlIndex( int windowID ) const
//-----------------------------------------------------------------------------
{
	return ( windowID >> 8 ) - 1;
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAbout(wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxBoxSizer* pTopDownSizer;
	wxDialog dlg(this, wxID_ANY, wxString(wxT("About wxPropView")));
	wxIcon icon(mvIcon_xpm);
	dlg.SetIcon( icon );
	pTopDownSizer = new wxBoxSizer(wxVERTICAL);
	wxStaticText *pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("configure tool for %s devices"), COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("(C) 2005 - %s by %s"), CURRENT_YEAR, COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("Version %s"), VERSION_STRING ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxT("Support contact: www.matrix-vision.de") );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("This tool has been written using wxWidgets (www.wxwidgets.org) and was compiled with version %d.%d.%d of this library"), wxMAJOR_VERSION, wxMINOR_VERSION, wxMINOR_VERSION ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxT("The property grid control is based on code written by Jaakko Salli") );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxT("The latest stable version can be obtained from http://wxpropgrid.sourceforge.net/cgi-bin/index") );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("The complete source of this application can be obtained by contacting %s"), COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	wxButton* pBtnOK = new wxButton(&dlg, wxID_OK, wxT("OK"));
	pBtnOK->SetDefault();
	pTopDownSizer->Add( pBtnOK, 0, wxALL | wxALIGN_RIGHT, 15);
	dlg.SetSizer( pTopDownSizer );
	pTopDownSizer->Fit( &dlg );
	dlg.ShowModal();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnActivateDisplay( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_pDisplayArea->SetActive( e.IsChecked() );
	m_pMISettings_Display_Active->Check( e.IsChecked() );
	GetToolBar()->ToggleTool( miSettingsDisplay_Active, e.IsChecked() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnBtnAbort( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	Abort();
}
//-----------------------------------------------------------------------------
void PropViewFrame::OnBtnRecord( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );

	if( pThread )
	{
		pThread->SetRecordMode(e.IsChecked());
		m_pMICapture_Record->Check( e.IsChecked() );
		GetToolBar()->ToggleTool( miCapture_Record, e.IsChecked() );
		SetupUpdateFrequencies( true );
	}
	SetupDlgControls();
}
//-----------------------------------------------------------------------------
void PropViewFrame::OnBtnForward( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );

	if( pThread )
	{
		SetupUpdateFrequencies( false );
		size_t nr = pThread->DisplaySequenceRequestNext();
		WriteLogMessage( wxString::Format( wxT("Recorded image %lu: "), (unsigned long int) nr ) );
		UpdateData( true );
	}
	SetupDlgControls();
}
//-----------------------------------------------------------------------------
void PropViewFrame::OnBtnBackward( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );

	if( pThread )
	{
		SetupUpdateFrequencies( false );
		size_t nr = pThread->DisplaySequenceRequestPrev();
		WriteLogMessage( wxString::Format( wxT("Recorded image %lu: "), (unsigned long int) nr ) );
		UpdateData( true );
	}
	SetupDlgControls();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnBtnLive( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	ConfigureLive();
}

//-----------------------------------------------------------------------------
void PropViewFrame::ToggleCurrentDevice( void )
//-----------------------------------------------------------------------------
{
	int devIndex = m_pDevCombo->GetSelection();
	Device* pDev = m_pDevPropHandler->GetActiveDevice();
	if( pDev )
	{
		bool isOpen = pDev->isOpen();
		WriteLogMessage( wxT("Trying to ") );
		WriteLogMessage( isOpen ? wxT("close"): wxT("open") );
		WriteLogMessage( wxString::Format( wxT(" device %d ("), devIndex ) );
		WriteLogMessage( ConvertedString(pDev->serial.read()) );
		WriteLogMessage( wxT(")\n") );
		try
		{
			if( isOpen )
			{
				m_pDevPropHandler->CloseDriver( m_pDevCombo->GetValue() );
				UpdateData();
			}
			else
			{
				m_pDevPropHandler->OpenDriver( m_pDevCombo->GetValue(), this );
			}
			WriteLogMessage( wxT(" Done...\n") );
		}
		catch( const ImpactAcquireException& e )
		{
			WriteErrorMessage( wxString::Format( wxT("\n%s(%s)\n"), ConvertedString(e.getErrorString()).c_str(), ConvertedString(e.getErrorCodeAsString()).c_str() ) );
			WriteErrorMessage( wxString::Format( wxT("Error origin: %s.\n"), ConvertedString(e.getErrorOrigin()).c_str() ) );
		}
		SetupDlgControls();
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnUseDevice( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	ToggleCurrentDevice();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnBtnSnap(wxCommandEvent& )
//-----------------------------------------------------------------------------
{

	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );
	if( pThread )
	{
		SetupUpdateFrequencies( false );
		int result = pThread->RequestSingle();
		if( result != DMR_NO_ERROR )
		{
			WriteErrorMessage( wxString::Format( wxT("Image request failed(Reason: %s)! More information can be found in the *.log-file or the debug output.\n"), ConvertedString(ImpactAcquireException::getErrorCodeAsString( result )).c_str() ) );
		}
		SetupDlgControls();
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnCBEnableAnalysisPlot( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	int plotIndex = GetAnalysisControlIndex( e.GetId() );
	m_ImageAnalysisPlots[plotIndex].m_pPlotCanvas->SetActive( e.IsChecked() );
	if( e.IsChecked() )
	{
		if( m_CurrentImage.getBuffer() )
		{
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIh->SetRange( 1, m_CurrentImage.getBuffer()->iHeight );
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIw->SetRange( 1, m_CurrentImage.getBuffer()->iWidth );
		}
		m_ImageAnalysisPlots[plotIndex].m_pPlotCanvas->RefreshData( m_CurrentImage.getBuffer(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIx->GetValue(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIy->GetValue(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIw->GetValue(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIh->GetValue(),
			true );
	}
	m_pDisplayArea->Refresh( false );
	m_ImageAnalysisPlots[plotIndex].UpdateControls();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotCBAOIFullMode( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	int plotIndex = GetAnalysisControlIndex( e.GetId() );
	m_ImageAnalysisPlots[plotIndex].m_pPlotCanvas->SetAOIFullMode( e.IsChecked() );
	if( m_ImageAnalysisPlots[plotIndex].m_pPlotCanvas->IsActive() )
	{
		m_ImageAnalysisPlots[plotIndex].m_pPlotCanvas->RefreshData( m_CurrentImage.getBuffer(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIx->GetValue(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIy->GetValue(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIw->GetValue(),
			m_ImageAnalysisPlots[plotIndex].m_pSCAOIh->GetValue(),
			true );
		m_pDisplayArea->Refresh( false );
	}
	m_ImageAnalysisPlots[plotIndex].UpdateControls();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnCBEnableInfoPlot( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	m_pInfoPlotArea->SetActive( m_pCBEnableInfoPlot->IsChecked() );
	if( m_pCBEnableInfoPlot->IsChecked() )
	{
		m_pInfoPlotArea->RefreshData( m_CurrentRequestInfo, true );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnClose( wxCloseEvent& )
//-----------------------------------------------------------------------------
{
	Deinit();
	Destroy();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnConnectedOnly( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	UpdateDeviceComboBox();
}

//-----------------------------------------------------------------------------
void PropViewFrame::UpdateData( bool boShowInfo )
//-----------------------------------------------------------------------------
{
	CaptureThread* pCT = 0;

	// To prevent a crash when the handler OnImageReady() is called and m_pDevPropHandler() is already deleted.
	if( m_pDevPropHandler == 0 )
	{
		return;
	}

	m_pDevPropHandler->GetActiveDevice( 0, 0, &pCT );
	TImageBufferPixelFormat lastPixelFormat = m_CurrentImage.getBuffer()->pixelFormat;
	bool boShowRequestInfoAsOverlay = m_pMISettings_Display_ShowRequestInfoAsOverlay->IsChecked();
	vector<wxString> requestInfoStrings;
	bool boFillInfoStringVector = boShowRequestInfoAsOverlay || boShowInfo;
	if( pCT )
	{
		pCT->GetImageData( &m_CurrentImage, &m_CurrentRequestInfo, m_CurrentPixelFormat, boFillInfoStringVector , requestInfoStrings );
		if( boShowInfo )
		{
			wxString msg;
			vector<wxString>::size_type vSize = requestInfoStrings.size();
			for( vector<wxString>::size_type i=0; i<vSize; i++ )
			{
				msg.append( requestInfoStrings[i] );
				msg.append( ( i < vSize - 1 ) ? wxT(", ") : wxT("\n") );
			}
			WriteLogMessage( msg );
		}
	}
	else
	{
		ImageBufferDesc nbd(1);
		m_CurrentImage = nbd;
		m_CurrentImage.getBuffer()->pixelFormat = ibpfMono8;
		//m_CurrentImage = ImageBufferDesc(ib); // doesn't work with GCC...
		m_CurrentPixelFormat = m_UNKNOWN_STRING;
		memset( &m_CurrentRequestInfo, 0, sizeof(m_CurrentRequestInfo) );
	}

	if( boShowRequestInfoAsOverlay )
	{
		m_pDisplayArea->SetInfoOverlay( requestInfoStrings );
	}
	ImageBuffer* p = m_CurrentImage.getBuffer();
	if( !m_pDisplayArea->SetImage( p ) && ( lastPixelFormat != p->pixelFormat ) )
	{
		WriteErrorMessage( wxString::Format( wxT("%s: ERROR!!! Don't know how to display format %s.\n"), __FUNCTION__, m_CurrentPixelFormat.c_str() ) );
	}
	if( p && p->vpData )
	{
#ifdef BUILD_WITH_PLUGIN_SUPPORT
		m_pluginManager.forwardImage( p );
#endif // #ifdef BUILD_WITH_PLUGIN_SUPPORT
		switch( p->pixelFormat )
		{
		case ibpfMono8:
		case ibpfMono10:
		case ibpfMono12:
		case ibpfMono14:
		case ibpfMono16:
		case ibpfRGBx888Packed:
		case ibpfRGB888Packed:
		case ibpfRGB101010Packed:
		case ibpfRGB121212Packed:
		case ibpfRGB141414Packed:
		case ibpfRGB161616Packed:
		case ibpfRGBx888Planar:
		case ibpfYUV422Packed:
		case ibpfYUV422Planar:
			if( m_ImageAnalysisPlots[0].m_pSCAOIw->GetMax() != p->iWidth )
			{
				for( unsigned int i=0; i<iapLAST; i++ )
				{
					m_ImageAnalysisPlots[i].m_pSCAOIw->SetRange( 1, p->iWidth );
					m_ImageAnalysisPlots[i].m_pSCAOIx->SetRange( 0, p->iWidth - 1 );
					// otherwise the limits are correct, but the displayed value might be incorrect
					m_ImageAnalysisPlots[i].m_pSCAOIw->SetValue( m_ImageAnalysisPlots[i].m_pSCAOIw->GetValue() );
					m_ImageAnalysisPlots[i].m_pSCAOIx->SetValue( m_ImageAnalysisPlots[i].m_pSCAOIx->GetValue() );
				}
			}
			if( m_ImageAnalysisPlots[0].m_pSCAOIh->GetMax() != p->iHeight )
			{
				for( unsigned int i=0; i<iapLAST; i++ )
				{
					m_ImageAnalysisPlots[i].m_pSCAOIh->SetRange( 1, p->iHeight );
					m_ImageAnalysisPlots[i].m_pSCAOIy->SetRange( 0, p->iHeight - 1 );
					// otherwise the limits are correct, but the displayed value might be incorrect
					m_ImageAnalysisPlots[i].m_pSCAOIh->SetValue( m_ImageAnalysisPlots[i].m_pSCAOIh->GetValue() );
					m_ImageAnalysisPlots[i].m_pSCAOIy->SetValue( m_ImageAnalysisPlots[i].m_pSCAOIy->GetValue() );
				}
			}
			for( unsigned int i=0; i<iapLAST; i++ )
			{
				if( m_ImageAnalysisPlots[i].m_pPlotCanvas->IsActive() )
				{
					m_ImageAnalysisPlots[i].m_pPlotCanvas->RefreshData( p, m_ImageAnalysisPlots[i].m_pSCAOIx->GetValue(), m_ImageAnalysisPlots[i].m_pSCAOIy->GetValue(), m_ImageAnalysisPlots[i].m_pSCAOIw->GetValue(), m_ImageAnalysisPlots[i].m_pSCAOIh->GetValue() );
				}
			}
			break;
		default:
			if( lastPixelFormat != p->pixelFormat )
			{
				WriteErrorMessage( wxString::Format( wxT("%s: ERROR!!! Don't know how to calculate data from format %s. Some of the image analysis plots can't be displayed.\n"), __FUNCTION__, m_CurrentPixelFormat.c_str() ) );
			}
			break;
		}
		if( lastPixelFormat != p->pixelFormat )
		{
			switch( p->pixelFormat )
			{
			case ibpfMono10:
			case ibpfMono12:
			case ibpfMono14:
			case ibpfMono16:
			case ibpfRGB101010Packed:
			case ibpfRGB121212Packed:
			case ibpfRGB141414Packed:
			case ibpfRGB161616Packed:
			case ibpfRGBx888Planar:
			case ibpfYUV422Packed:
			case ibpfYUV422Planar:
				WriteLogMessage( wxString::Format( wxT("%s: Please note that pixel format %s will require additional CPU time as it needs to be converted for displaying.\n"), __FUNCTION__, m_CurrentPixelFormat.c_str() ) );
				break;
			default:
				break;
			}
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::UpdateDeviceFromComboBox( void )
//-----------------------------------------------------------------------------
{
	m_pDevPropHandler->SetActiveDevice( m_pDevCombo->GetValue() );
	UpdateTitle();
	UpdateData();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnDevComboTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	UpdateDeviceFromComboBox();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnEventNotified( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	Device* pDevice = static_cast<Device*>(e.GetClientData());
	if( pDevice )
	{
		WriteLogMessage( wxString::Format( wxT("There has been a %s event reported by device %s.\n"), e.GetString().c_str(), ConvertedString(pDevice->serial.read()).c_str() ), wxTextAttr(wxColour(0, 150, 50)) );
	}
	e.Skip();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnExportActiveDevice( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxFileDialog fileDlg( this, wxT("Select a filename"), wxT(""), wxT(""), wxT("mv-settings Files (*.xml)|*.xml"), wxSAVE | wxOVERWRITE_PROMPT );
	if( fileDlg.ShowModal() == wxID_OK )
	{
		wxString pathName = fileDlg.GetPath();
		FunctionInterface* p = 0;
		m_pDevPropHandler->GetActiveDevice( &p );
		if( p )
		{
			string pathANSI(pathName.mb_str());
			int result = p->saveSetting( pathANSI, sfDefault, sGlobal );
			if( result == 0 )
			{
				WriteLogMessage( wxString::Format( wxT("Storing of data to %s successful.\n"), pathName.c_str() ) );
			}
			else
			{
				WriteErrorMessage( wxString::Format( wxT("Storing of data to %s failed(error code: %d(%s)).\n"), pathName.c_str(), result, ConvertedString(ImpactAcquireException::getErrorCodeAsString(result)).c_str() ) );
			}
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnFitToDisplay( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_pDisplayArea->SetScaling( e.IsChecked() );
	m_pMISettings_Display_FitToScreen->Check( e.IsChecked() );
	GetToolBar()->ToggleTool( miSettingsDisplay_FitToScreen, e.IsChecked() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotDrawStartChanged( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	dynamic_cast<HistogramCanvas*>(m_ImageAnalysisPlots[GetAnalysisControlIndex( e.GetId() )].m_pPlotCanvas)->SetDrawStartPercent( e.GetInt() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotDrawWindowChanged( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	dynamic_cast<HistogramCanvas*>(m_ImageAnalysisPlots[GetAnalysisControlIndex( e.GetId() )].m_pPlotCanvas)->SetDrawWindowWidthPercent( e.GetInt() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotUpdateSpeedTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	if( m_pDevPropHandler )
	{
		m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );
		if( pThread )
		{
			if( pThread->GetLiveMode() )
			{
				SetupUpdateFrequencies( true );
			}
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnImageInfo( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	auto_ptr<RequestInfoData> p(reinterpret_cast<RequestInfoData*>(e.GetClientData()));
	if( m_pInfoPlotArea )
	{
		m_pInfoPlotArea->RefreshData( *p );
	}
	e.Skip();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnImageReady( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	UpdateData();
	e.Skip();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnImageSkipped( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );
	size_t cnt = 1;
	if( pThread )
	{
		cnt = pThread->GetSkippedImageCount();
	}
	if( m_pLogWindow && m_pDisplayArea && m_pDisplayArea->IsActive() )
	{
		m_pDisplayArea->IncreaseSkippedCounter(cnt);
	}
	e.Skip();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSequenceReady( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );

	if( pThread && m_pLogWindow )
	{
		wxString msg(wxString::Format( wxT("All %lu ('RequestCount') images recorded.\n"), (unsigned long int)(pThread->GetSequenceImageCnt()) ));
		WriteLogMessage( msg );
		wxMessageDialog dlg( NULL, msg, wxT("Info"), wxOK | wxICON_WARNING );
		dlg.ShowModal();

	}
	e.Skip();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnImportActiveDevice( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxFileDialog fileDlg( this, wxT("Select a filename"), wxT(""), wxT(""), wxT("mv-settings Files (*.xml)|*.xml"), wxOPEN | wxFILE_MUST_EXIST );
	if( fileDlg.ShowModal() == wxID_OK )
	{
		wxString pathName = fileDlg.GetPath();
		FunctionInterface* p = 0;
		m_pDevPropHandler->GetActiveDevice( &p );
		if( p )
		{
			string pathANSI(pathName.mb_str());
			int result = p->loadSetting( pathANSI, sfDefault, sGlobal );
			if( result == 0 )
			{
				WriteLogMessage( wxString::Format( wxT("Importing data from %s successful.\n"), pathName.c_str() ) );
			}
			else
			{
				WriteLogMessage( wxString::Format( wxT("Importing data from %s failed(error code: %d(%s)).\n"), pathName.c_str(), result, ConvertedString(ImpactAcquireException::getErrorCodeAsString( result )).c_str() ) );
			}
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnPopUpMethExec( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxPGId id = m_pPropGrid->GetSelectedProperty();
	if( wxPGIdIsOk(id) )
	{
		WriteLogMessage( static_cast<MethodObject*>(wxPGIdToPtr(id)->GetClientData())->Call() );
		WriteLogMessage( wxT('\n') );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotAOIxTextChanged( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_ImageAnalysisPlots[GetAnalysisControlIndex( e.GetId() )].m_pPlotCanvas->RefreshData( m_CurrentImage.getBuffer(), e.GetInt() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotAOIyTextChanged( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_ImageAnalysisPlots[GetAnalysisControlIndex( e.GetId() )].m_pPlotCanvas->RefreshData( m_CurrentImage.getBuffer(), -1, e.GetInt() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotAOIwTextChanged( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_ImageAnalysisPlots[GetAnalysisControlIndex( e.GetId() )].m_pPlotCanvas->RefreshData( m_CurrentImage.getBuffer(), -1, -1, e.GetInt() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnAnalysisPlotAOIhTextChanged( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_ImageAnalysisPlots[GetAnalysisControlIndex( e.GetId() )].m_pPlotCanvas->RefreshData( m_CurrentImage.getBuffer(), -1, -1, -1, e.GetInt() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnInfoPlot_PlotSelectionComboTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	InfoPlotCanvas::TPlotSelection plotSelection = static_cast<InfoPlotCanvas::TPlotSelection>(wxPtrToUInt( m_pInfoPlotSelectionCombo->GetClientData( m_pInfoPlotSelectionCombo->GetSelection() ) ));
	m_pInfoPlotArea->SetPlotSelection( plotSelection );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnInfoPlotUpdateHistoryDepth( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_pInfoPlotArea->SetHistoryDepth( e.GetInt() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnInfoPlotUpdateSpeedTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	m_pInfoPlotArea->SetUpdateFrequency( m_pSCInfoPlotUpdateSpeed->GetValue() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnLiveModeAborted( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	wxString msg(wxString::Format( wxT("The live mode has been aborted because of the following error: '%s'.\n"), e.GetString().c_str() ));
	wxMessageDialog dlg( NULL, msg, wxT("Information"), wxOK | wxICON_INFORMATION );
	dlg.ShowModal();
	WriteErrorMessage( msg );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnLoadActiveDevice( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	if( m_pDevPropHandler )
	{
		FunctionInterface* p = 0;
		m_pDevPropHandler->GetActiveDevice( &p );
		if( p )
		{
			string serialANSI(m_pDevCombo->GetValue().mb_str());
			p->loadSetting( serialANSI, sfNative, sUser );
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnLoadFromDefault( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	if( m_pDevPropHandler )
	{
		FunctionInterface* p = 0;
		m_pDevPropHandler->GetActiveDevice( &p );
		if( p )
		{
			p->loadSettingFromDefault();
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnPopUpPropAppendValue( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxPGId id = m_pPropGrid->GetSelectedProperty();
	if( wxPGIdIsOk(id) )
	{
		PropData* pData = static_cast<PropData*>(wxPGIdToPtr(id)->GetClientData());
		if( pData )
		{
			VectorPropertyObject* pProp = dynamic_cast<VectorPropertyObject*>(pData);
			if( pProp )
			{
				Property prop(pProp->Component().hObj());
				prop.resizeValArray( prop.valCount() + 1 );
			}
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnPopUpPropDeleteValue( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxPGId id = m_pPropGrid->GetSelectedProperty();
	if( wxPGIdIsOk(id) )
	{
		PropData* pData = static_cast<PropData*>(wxPGIdToPtr(id)->GetClientData());
		if( pData )
		{
			VectorPropertyObject* pProp = dynamic_cast<VectorPropertyObject*>(pData);
			if( pProp )
			{
				Property prop(pProp->Component().hObj());
				pProp->RemoveValue( prop.valCount() - 1 );
			}
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnPopUpPropRestoreDefault( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxPGId id = m_pPropGrid->GetSelectedProperty();
	if( wxPGIdIsOk(id) )
	{
		Component comp = static_cast<PropData*>(wxPGIdToPtr(id)->GetClientData())->Component();
		if( comp.type() == ctList )
		{
			wxMessageDialog AreYouSureDlg( NULL,
				wxString::Format( wxT("All properties in the list '%s' will be set to default.\n"), ConvertedString(comp.name()).c_str() ),
				wxT("Warning"),
				wxNO_DEFAULT | wxYES_NO | wxICON_INFORMATION );

			if( AreYouSureDlg.ShowModal() != wxID_YES )
			{
				return;
			}
		}
		if( comp.isDefault() )
		{
			WriteLogMessage( wxString::Format( wxT("Element '%s' is already set to the default value.\n"), ConvertedString(comp.name()).c_str() ) );
		}
		else
		{
			comp.restoreDefault();
		}
		m_pPropGrid->ClearModifiedStatus( id );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnPropertyChanged( wxPropertyGridEvent& e )
//-----------------------------------------------------------------------------
{
	PropData* prop_data = static_cast<PropData*>(wxPGIdToPtr(e.GetProperty())->GetClientData());

	if( prop_data )
	{
		prop_data->UpdatePropData();
	}
	m_pDevPropHandler->SetActiveDevice( m_pDevCombo->GetValue() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnPropertyRightClicked( wxPropertyGridEvent& e )
//-----------------------------------------------------------------------------
{
	PropData* prop_data = static_cast<PropData*>(wxPGIdToPtr(e.GetProperty())->GetClientData());

	if( prop_data )
	{
		TComponentType type = prop_data->Component().type();
		if( ( ( type & ctProp ) || ( type & ctList ) ) && prop_data->Component().isWriteable() )
		{
			wxMenu menu(wxT(""));
			menu.Append( miPopUpPropRestoreDefault, wxT("&Restore default") );
			TComponentFlag flags = prop_data->Component().flags();
			wxPGId id = m_pPropGrid->GetSelectedProperty();
			if( wxPGIdIsOk(id) )
			{
				PropData* pData = static_cast<PropData*>(wxPGIdToPtr(id)->GetClientData());
				if( pData )
				{
					VectorPropertyObject* pProp = dynamic_cast<VectorPropertyObject*>(pData);
					if( pProp )
					{
						if( !( flags & cfFixedSize ) )
						{
							Property driverProp(pProp->Component());
							unsigned int valCount = driverProp.valCount();
							unsigned int maxValCount = driverProp.maxValCount();
							wxMenuItem* pMI = menu.Append( miPopUpPropAppendValue, wxT("&Append value") );
							if( maxValCount <= valCount )
							{
								pMI->Enable( false );
							}
							pMI = menu.Append( miPopUpPropDeleteValue, wxT("&Delete last value") );
							if( valCount <= 1 )
							{
								pMI->Enable( false );
							}
						}
					}
				}
			}
			PopupMenu( &menu );
		}
		/// \todo disable read-only methods later?
		else if( ( type & ctMeth ) /* && prop_data->Component().isWriteable()*/ )
		{
			wxMenu menu(wxT(""));
			menu.Append(miPopUpMethExec, wxT("&Execute"));
			PopupMenu(&menu);
		}
	}
	m_pDevPropHandler->SetActiveDevice( m_pDevCombo->GetValue() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnQuit( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	// true is to force the frame to close
	Close( true );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSaveActiveDevice( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	FunctionInterface* p = 0;
	m_pDevPropHandler->GetActiveDevice( &p );
	if( p )
	{
		string pathANSI(m_pDevCombo->GetValue().mb_str());
		p->saveSetting( pathANSI, sfNative, sUser );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::SaveImage( const wxString& filenameAndPath, const wxString& extension )
//-----------------------------------------------------------------------------
{
	if( extension == wxT("raw") )
	{
		wxFileName tmpFilename(filenameAndPath);
		wxString name(tmpFilename.GetName());
		tmpFilename.SetName( wxString::Format( wxT("%s_%dx%d_%s"), name.c_str(), m_CurrentImage.getBuffer()->iWidth, m_CurrentImage.getBuffer()->iHeight, m_CurrentPixelFormat.c_str() ) );
		wxFile file(tmpFilename.GetFullPath().c_str(), wxFile::write);
		if( !file.IsOpened() )
		{
			WriteErrorMessage( wxString::Format( wxT("Storing of %s failed. Could not open file.\n"), tmpFilename.GetFullPath().c_str() ) );
			return;
		}
		size_t bytesWritten = file.Write( m_CurrentImage.getBuffer()->vpData, m_CurrentImage.getBuffer()->iSize );
		if( bytesWritten < static_cast<size_t>(m_CurrentImage.getBuffer()->iSize) )
		{
			WriteErrorMessage( wxString::Format( wxT("Storing of %s failed. Could not write all data.\n"), tmpFilename.GetFullPath().c_str() ) );
			return;
		}
		WriteLogMessage( wxString::Format( wxT("Storing of %s successful.\n"), tmpFilename.GetFullPath().c_str() ) );
	}
	else
	{
		ImageCanvas::TSaveResult result = m_pDisplayArea->SaveCurrentImage( filenameAndPath, extension );
		switch( result )
		{
		case ImageCanvas::srOK:
			WriteLogMessage( wxString::Format( wxT("Storing of %s successful.\n"), filenameAndPath.c_str() ) );
			break;
		case ImageCanvas::srNoImage:
			{
				wxMessageDialog errorDlg( NULL, wxT("There is no valid image to store!"), wxT("Error"), wxOK | wxICON_INFORMATION );
				errorDlg.ShowModal();
			}
			break;
		case ImageCanvas::srFailedToSave:
			WriteErrorMessage( wxString::Format( wxT("Storing of %s failed.\n"), filenameAndPath.c_str() ) );
			break;
		default:
			WriteLogMessage( wxString::Format( wxT("Storing of %s return unexpected value: %d.\n"), filenameAndPath.c_str(), result ) );
			break;
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSaveImageSequence( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	CaptureThread* pThread = 0;
	m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );

	if( pThread )
	{
		wxFileDialog fileDlg( this, wxT("Select a filename"), wxT(""), wxT(""), wxT("BMP files (*.bmp)|*.bmp|PNG files (*.png)|*.png|RAW files (*.raw)|*.raw"), wxSAVE );
		if( fileDlg.ShowModal() == wxID_OK )
		{
			wxString fileName = fileDlg.GetPath();
			if( fileName.IsEmpty() )
			{
				return;
			}
			wxFileName tmpFilename( fileName );
			wxString extension, name;
			wxFileName::SplitPath( fileName, 0, &name, &extension );

			// remove number
			if( name.Len() > 3 )
			{
				wxString numstr = name.Right( 3 );
				if( numstr.IsNumber() )
				{
					name.RemoveLast( 3 );
				}
			}

			const vector<int> &vSeq = pThread->GetSequenceImageVector();
			vector<int>::size_type vSize = vSeq.size();
			for( vector<int>::size_type i=0; i<vSize; i++ )
			{
				pThread->DisplaySequenceRequest( i );
				UpdateData();
				tmpFilename.SetName( wxString::Format( wxT("%s%03lu"), name.c_str(), (unsigned long int) i ) );
				SaveImage( tmpFilename.GetFullPath(), extension );
			}
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSaveImage( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxFileDialog fileDlg( this, wxT("Select a filename"), wxT(""), wxT(""), wxT("BMP files (*.bmp)|*.bmp|PNG files (*.png)|*.png|RAW files (*.raw)|*.raw"), wxSAVE | wxOVERWRITE_PROMPT );
	if( fileDlg.ShowModal() == wxID_OK )
	{
		wxString fileName = fileDlg.GetPath();
		if( fileName.IsEmpty() )
		{
			return;
		}

		wxString extension;
		wxFileName::SplitPath( fileName, 0, 0, &extension );
		SaveImage( fileName, extension );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSaveToDefault( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	FunctionInterface* p = 0;
	m_pDevPropHandler->GetActiveDevice( &p );
	if( p )
	{
		p->saveSettingToDefault();
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSaveSystemToDefault( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	FunctionInterface* p = 0;
	m_pDevPropHandler->GetActiveDevice( &p );
	if( p )
	{
		p->saveSystemToDefault();
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnShowDisplayMessagesAsOverlay(wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_pDisplayArea->SetPerformanceWarningOutput( e.IsChecked() );
	m_pMISettings_Display_ShowMessagesAsOverlay->Check( e.IsChecked() );
	GetToolBar()->ToggleTool( miSettingsDisplay_OverlayMessages, e.IsChecked() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnShowRequestInfosAsOverlay( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	m_pDisplayArea->SetInfoOverlayMode( e.IsChecked() );
	m_pMISettings_Display_ShowRequestInfoAsOverlay->Check( e.IsChecked() );
	GetToolBar()->ToggleTool( miSettingsDisplay_RequestInfoAsOverlay, e.IsChecked() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSettingsDisplayLogWindow( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	SetupDisplayLogSplitter( e.IsChecked() );
	m_pMISettings_Display_LogWindow->Check( e.IsChecked() );
	GetToolBar()->ToggleTool( miSettings_ShowLogWindow, e.IsChecked() );
	m_pDisplayArea->Refresh( true );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSettingsPropGrid_Show( wxCommandEvent& e )
//-----------------------------------------------------------------------------
{
	SetupVerSplitter( e.IsChecked() );
	m_pMISettings_PropGrid_Show->Check( e.IsChecked() );
	GetToolBar()->ToggleTool( miSettingsPropGrid_Show, e.IsChecked() );
	m_pDisplayArea->Refresh( true );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSettingsPropGrid_StandardView( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	m_pDevPropHandler->SetViewMode( DevicePropertyHandler::vmStandard );
	m_ViewMode = m_pDevPropHandler->GetViewMode();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSettingsPropGrid_DevelopersView( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	m_pDevPropHandler->SetViewMode( DevicePropertyHandler::vmDeveloper );
	m_ViewMode = m_pDevPropHandler->GetViewMode();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSize( wxSizeEvent& e )
//-----------------------------------------------------------------------------
{
	if( m_pDisplayArea )
	{
		m_pDisplayArea->Refresh( false );
	}
	for( unsigned int i=0; i<iapLAST; i++ )
	{
		if( m_ImageAnalysisPlots[i].m_pPlotCanvas )
		{
			m_ImageAnalysisPlots[i].m_pPlotCanvas->Refresh( false );
		}
	}
	e.Skip();
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnUpdateDeviceList( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	m_pDevPropHandler->GetDevMgr().updateDeviceList();
}

//-----------------------------------------------------------------------------
void PropViewFrame::UpdatePropGridSplitter( void )
//-----------------------------------------------------------------------------
{
	double ratio = m_pPropGrid->GetSplitterRatio();
	if( ratio != -1 )
	{
		m_pVerticalSplitter->UpdateSize();
		wxRect rect = m_pPropGrid->GetRect();
		m_pPropGrid->SetSplitterPosition( (int)( rect.GetWidth()* ratio ), true );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnSplitterSashPosChanged( wxSplitterEvent& )
//-----------------------------------------------------------------------------
{
	UpdatePropGridSplitter();
	m_pDisplayArea->Refresh( true );
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnTimer( wxTimerEvent& e )
//-----------------------------------------------------------------------------
{
	switch( e.GetId() )
	{
	case teListUpdate:
		if( m_pPropGrid->IsShown() )
		{
			// while the propgrid is not displayed we don't need to update it
			m_pDevPropHandler->ValidateTrees();
		}
		SetupDlgControls();
		UpdateStatusBar();
		break;
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::OnToggleDisplayWindowSize( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	if( !m_pHorizontalSplitter->IsSplit() && !m_pVerticalSplitter->IsSplit() )
	{
		// if all hideable windows are hidden already restore them
		m_boDisplayWindowMaximized = false;
	}
	else
	{
		// else toggle current status
		m_boDisplayWindowMaximized = !m_boDisplayWindowMaximized;
	}

	if( m_pMISettings_Display_LogWindow->IsChecked() || !m_boDisplayWindowMaximized )
	{
		SetupDisplayLogSplitter( !m_boDisplayWindowMaximized );
	}

	if( m_pMISettings_PropGrid_Show->IsChecked() || !m_boDisplayWindowMaximized )
	{
		SetupVerSplitter( !m_boDisplayWindowMaximized );
	}
	m_pMISettings_PropGrid_Show->Check( !m_boDisplayWindowMaximized );
	m_pMISettings_Display_LogWindow->Check( !m_boDisplayWindowMaximized );
	GetToolBar()->ToggleTool( miSettings_ShowLogWindow, !m_boDisplayWindowMaximized );
	GetToolBar()->ToggleTool( miSettingsPropGrid_Show, !m_boDisplayWindowMaximized );
	m_pDisplayArea->RefreshScrollbars();
	m_pDisplayArea->Refresh( true );
}

//-----------------------------------------------------------------------------
void PropViewFrame::SetupDlgControls( void )
//-----------------------------------------------------------------------------
{
	const Device* pDev = m_pDevPropHandler->GetActiveDevice();
	bool boDevOpen = false;
	bool boDevPresent = false;
	bool boDevLive = false;
	bool boSequenceBegin = true;
	bool boSequenceEnd = true;
	bool boDevRecord = false;
	if( pDev )
	{
		boDevOpen = pDev->isOpen();
		boDevPresent = ( pDev->state.read() == 1 );
		CaptureThread* pThread = 0;
		m_pDevPropHandler->GetActiveDevice( 0, 0, &pThread );
		if( pThread )
		{
			boDevRecord = pThread->GetRecordMode();
			boDevLive = pThread->GetLiveMode();
			if( pThread->GetSequenceImageCnt() > 1 )
			{
				boSequenceBegin = pThread->GetCurrentSequenceIndex() == 0;
				boSequenceEnd = pThread->GetCurrentSequenceIndex() == pThread->GetSequenceImageCnt() - 1;
			}
		}
	}

	const DeviceManager& devMgr = m_pDevPropHandler->GetDevMgr();
	if( m_DeviceListChangedCounter != devMgr.changedCount() )
	{
		UpdateDeviceComboBox();
	}

	bool isDeviceSelected = m_pDevCombo->GetValue() != m_NoDevStr;
	m_pMIAction_Use->Enable( isDeviceSelected );
	m_pMIAction_Use->Check( boDevOpen );
	m_pMICapture_Record->Check( boDevRecord );

	m_pMIAction_Save_SystemToDefault->Enable( boDevOpen );
	m_pMIAction_Save_SystemToDefault->Enable( boDevOpen );
	m_pMIAction_Save_ActiveDevice->Enable( boDevOpen );
	m_pMIAction_Save_ExportActiveDevice->Enable( boDevOpen );
	m_pMIAction_Load_FromDefault->Enable( boDevOpen );
	m_pMIAction_Load_FromDefault->Enable( boDevOpen );
	m_pMIAction_Load_ImportActiveDevice->Enable( boDevOpen );
	m_pMIAction_SaveImage->Enable( boDevOpen && !boDevLive );
	m_pMIAction_SaveImageSequence->Enable( boDevOpen && !boDevLive && (!boSequenceEnd || !boSequenceBegin) );
	m_pMICapture_Live->Enable( boDevOpen && boDevPresent );
	m_pMICapture_Live->Check( boDevLive );
	m_pMICapture_Snap->Enable( boDevOpen && boDevPresent && !boDevLive );
	m_pMICapture_Abort->Enable( boDevOpen && boDevPresent );
	m_pMICapture_Record->Enable( boDevOpen && boDevPresent && !boDevLive  );
	m_pMICapture_Forward->Enable( boDevOpen && boDevPresent && !boDevLive  && !boSequenceEnd );
	m_pMICapture_Backward->Enable( boDevOpen && boDevPresent && !boDevLive && !boSequenceBegin );

	m_pToolBar->EnableTool( miAction_UseDevice, isDeviceSelected );
	m_pToolBar->ToggleTool( miAction_UseDevice, boDevOpen );
	m_pToolBar->EnableTool( miCapture_Abort, m_pMICapture_Abort->IsEnabled() );
	m_pToolBar->EnableTool( miCapture_Live, m_pMICapture_Live->IsEnabled() );
	m_pToolBar->ToggleTool( miCapture_Live, boDevLive );
	m_pToolBar->EnableTool( miCapture_Snap, m_pMICapture_Snap->IsEnabled() );
	m_pToolBar->EnableTool( miCapture_Record, m_pMICapture_Record->IsEnabled() );
	m_pToolBar->ToggleTool( miCapture_Record, boDevRecord );
	m_pToolBar->EnableTool( miCapture_Forward, m_pMICapture_Forward->IsEnabled() );
	m_pToolBar->EnableTool( miCapture_Backward, m_pMICapture_Backward->IsEnabled() );
}

//-----------------------------------------------------------------------------
void PropViewFrame::SetupDisplayLogSplitter( bool boSplit )
//-----------------------------------------------------------------------------
{
	if( boSplit )
	{
		m_pHorizontalSplitter->SplitHorizontally( m_pDisplayArea, m_pLowerRightWindow, 0 );
		m_pHorizontalSplitter->SetSashPosition( m_HorizontalSplitterPos );
	}
	else
	{
		m_HorizontalSplitterPos = m_pHorizontalSplitter->GetSashPosition();
		m_pHorizontalSplitter->Unsplit(m_pLowerRightWindow);
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::SetupUpdateFrequencies( bool boCurrentValue )
//-----------------------------------------------------------------------------
{
	for( unsigned int i=0; i<iapLAST; i++ )
	{
		if( m_ImageAnalysisPlots[i].m_pPlotCanvas )
		{
			m_ImageAnalysisPlots[i].m_pPlotCanvas->SetUpdateFrequency( ( boCurrentValue ) ? m_ImageAnalysisPlots[i].m_pSCUpdateSpeed->GetValue() : 1 );
		}
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::SetupVerSplitter( bool boSplit )
//-----------------------------------------------------------------------------
{
	if( boSplit )
	{
		m_pVerticalSplitter->SplitVertically( m_pPropGrid, m_pHorizontalSplitter, 0 );
		m_pVerticalSplitter->SetSashPosition( m_VerticalSplitterPos );		
	}
	else
	{
		m_VerticalSplitterPos = m_pVerticalSplitter->GetSashPosition();
		m_pVerticalSplitter->Unsplit(m_pPropGrid);
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::UpdateDeviceComboBox( void )
//-----------------------------------------------------------------------------
{
	const DeviceManager& devMgr = m_pDevPropHandler->GetDevMgr();
	bool boDisplayConnectedOnly = m_pMISettings_ShowConnectedOnly->IsChecked();
	m_DeviceListChangedCounter = devMgr.changedCount();
	int oldIndex = m_pDevCombo->GetSelection();
	wxString oldSelection = ( oldIndex == wxNOT_FOUND ) ? m_NoDevStr : m_pDevCombo->GetString( oldIndex );
	// delete all entries
	m_pDevCombo->Clear();
	// refill the combo box with up to date data
	unsigned int devCnt = devMgr.deviceCount();
	if( devCnt > 0 )
	{
		for( unsigned int i=0; i<devCnt; i++ )
		{
			if( !boDisplayConnectedOnly || ( devMgr[i]->state.read() == dsPresent ) )
			{
				m_pDevCombo->Append(ConvertedString(devMgr[i]->serial.read()));
			}
		}
		if( oldSelection == m_NoDevStr )
		{
			m_pDevCombo->Select(0);
		}
		else
		{
			int selection = m_pDevCombo->FindString( oldSelection );
			m_pDevCombo->Select( ( selection == wxNOT_FOUND ) ? 0 : selection );
		}
	}
	else
	{
		m_pDevCombo->Append(m_NoDevStr);
		m_pDevCombo->Select(0);
	}

	if( m_pDevCombo->GetString(0) != m_NoDevStr )
	{
		m_pDevPropHandler->SetActiveDevice( m_pDevCombo->GetValue() );
	}
	UpdateTitle();
	UpdateData();
}

//-----------------------------------------------------------------------------
void PropViewFrame::UpdateStatusBar( void )
//-----------------------------------------------------------------------------
{
	const Statistics* pS = 0;
	FunctionInterface* pFI = 0;
	CaptureThread* pCT = 0;
	m_pDevPropHandler->GetActiveDevice( &pFI, &pS, &pCT );

	if( pFI && pS && pCT )
	{
		if( m_pStatusBar->GetFieldsCount() != NO_OF_STATUSBAR_FIELDS )
		{
			int statusBarFieldWidths[NO_OF_STATUSBAR_FIELDS];
			statusBarFieldWidths[sbfFPS] = -2;
			statusBarFieldWidths[sbfFrameCount] = -2;
			statusBarFieldWidths[sbfImageFormat] = -5;
			statusBarFieldWidths[sbfErrorCount] = -2;
			statusBarFieldWidths[sbfExposeTime] = -3;
			statusBarFieldWidths[sbfGain] = -3;
			m_pStatusBar->SetFieldsCount( NO_OF_STATUSBAR_FIELDS, &statusBarFieldWidths[0] );
		}

		bool boSlowDisplay = ( m_CurrentImage.getBuffer()->pixelFormat != ibpfMono8 ) &&
							 ( m_CurrentImage.getBuffer()->pixelFormat != ibpfRGBx888Packed ) &&
							 ( m_CurrentImage.getBuffer()->pixelFormat != ibpfRGB888Packed ) &&
							 ( m_CurrentPixelFormat != m_UNKNOWN_STRING ); 
		m_pStatusBar->SetStatusText( wxString::Format(wxT("Frames/sec.: %.1f"), pS->framesPerSecond.read() ), sbfFPS );
		m_pStatusBar->SetStatusText( wxString::Format(wxT("Frame count: %d"), pS->frameCount.read() ), sbfFrameCount );
		wxString imageFormatMsg(wxString::Format(wxT("Image format %dx%d, %s%s (%s)"),
			m_CurrentImage.getBuffer()->iWidth,
			m_CurrentImage.getBuffer()->iHeight,
			m_CurrentPixelFormat.c_str(),
			( boSlowDisplay ? wxT("(slow display because of conversion)"): wxT("") ),
			pCT->GetLastRequestResult().c_str() ));
		m_pStatusBar->SetStatusText( imageFormatMsg, sbfImageFormat );
		m_pStatusBar->SetStatusText( wxString::Format(wxT("Error count: %d"), pS->errorCount.read() ), sbfErrorCount );
		m_pStatusBar->SetStatusText( wxString::Format(wxT("Expose time(us): %d"), m_CurrentRequestInfo.exposeTime_us ), sbfExposeTime );
		m_pStatusBar->SetStatusText( wxString::Format(wxT("Gain(dB): %.4f"), m_CurrentRequestInfo.gain_dB ), sbfGain );
	}
	else
	{
		if( m_pStatusBar->GetFieldsCount() != 1 )
		{
			m_pStatusBar->SetFieldsCount( 1, 0 );
		}
		m_pStatusBar->SetStatusText( wxT("no statistics available"), 0 );
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::UpdateTitle( void )
//-----------------------------------------------------------------------------
{
	wxString title(wxString::Format( wxT("wxPropView(%s) [%s]"), VERSION_STRING, m_pDevCombo->GetValue().c_str() ));
	if( m_pDevPropHandler )
	{
		string serialANSI(m_pDevCombo->GetValue().mb_str());
		Device* p = m_pDevPropHandler->GetDevMgr().getDeviceBySerial( serialANSI );
		if( p )
		{
			title.append( wxString::Format( wxT(", %s"), ConvertedString(p->product.read()).c_str() ) );
		}
	}
	SetTitle( title );
}

//-----------------------------------------------------------------------------
void PropViewFrame::WriteLogMessage( const wxString& msg )
//-----------------------------------------------------------------------------
{
	WriteLogMessage( msg, wxTextAttr( wxColour(0, 0, 0)));
}

//-----------------------------------------------------------------------------
void PropViewFrame::WriteLogMessage( const wxString& msg, const wxTextAttr& style )
//-----------------------------------------------------------------------------
{
	if( m_pLogWindow )
	{
		// If you want the control to show the last line of text at the bottom, you can add "ScrollLines(1)" 
		// right after the AppendText call. AppendText will ensure the new line is visible, and ScrollLines 
		// will ensure the scroll bar is at the real end of the range, not further.
		long posBefore = m_pLogWindow->GetLastPosition();
		m_pLogWindow->AppendText( msg );
		long posAfter = m_pLogWindow->GetLastPosition();
		m_pLogWindow->SetStyle( posBefore, posAfter, style );
		m_pLogWindow->ScrollLines( 1 );
		m_pLogWindow->ShowPosition( m_pLogWindow->GetLastPosition() ); // ensure ! that this position is really visible
	}
}

//-----------------------------------------------------------------------------
void PropViewFrame::WriteErrorMessage( const wxString& msg )
//-----------------------------------------------------------------------------
{
	wxTextAttr errorStyle( wxColour(255, 0, 0) );
	WriteLogMessage( msg, errorStyle );
}
