/*
 * (C) Copyright 2004, Psyent Corporation <www.psyent.com>
 * Scott McNutt <smcnutt@psyent.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

//#include <common.h>

//#if defined(CFG_NIOS_EPCSBASE)
//#include <command.h>
//#include <asm/io.h>
//#include <nios2-io.h>
//#include <nios2-epcs.h>
#include <linux/jiffies.h>
#include <asm/io.h>
#include <asm-cris/fasttimer.h>
#include "epcs.h"

/*-----------------------------------------------------------------------*/
#define SHORT_HELP\
	"epcs    - read/write Cyclone EPCS configuration device.\n"

#define LONG_HELP\
	"\n"\
	"epcs erase start [end]\n"\
	"    - erase sector start or sectors start through end.\n"\
	"epcs info\n"\
	"    - display EPCS device information.\n"\
	"epcs protect on | off\n"\
	"    - turn device protection on or off.\n"\
	"epcs read addr offset count\n"\
	"    - read count bytes from offset to addr.\n"\
	"epcs write addr offset count\n"\
	"    - write count bytes to offset from addr.\n"\
	"epcs verify addr offset count\n"\
	"    - verify count bytes at offset from addr.\n"


//Operation codes for serial configuration devices
#define EPCS_WRITE_ENA		0x06	/* Write enable */
#define EPCS_WRITE_DIS		0x04	/* Write disable */
#define EPCS_READ_STAT		0x05	/* Read status */
#define EPCS_READ_BYTES		0x03	/* Read bytes */
#define EPCS_READ_ID		0xab	/* Read silicon id */
#define EPCS_WRITE_STAT		0x01	/* Write status */
#define EPCS_WRITE_BYTES	0x02	/* Write bytes */
#define EPCS_ERASE_BULK		0xc7	/* Erase entire device */
#define EPCS_ERASE_SECT		0xd8	/* Erase sector */

// Device status register bits
#define EPCS_STATUS_WIP		(1<<0)	/* Write in progress */
#define EPCS_STATUS_WEL		(1<<1)	/* Write enable latch */

#define EPCS_TIMEOUT		25	/* 100 msec timeout */

//static nios_spi_t *epcs = (nios_spi_t *)CFG_NIOS_EPCSBASE;
static nios_spi_t *epcs = 0;

//-----------------------------------------------------------------------------
// Device access
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
int epcs_cs (int assert)
//-----------------------------------------------------------------------------
{
	unsigned tmp;
	unsigned long start_jiffie;

	if (assert) {
		tmp = ioread32( (void __iomem *)(&epcs->control) );
		iowrite32( tmp | NIOS_SPI_SSO, (void __iomem *)(&epcs->control) );
	} else {
		/* Let all bits shift out */
		start_jiffie = jiffies;
		while( (ioread32( (void __iomem *)(&epcs->status) ) & NIOS_SPI_TMT) == 0 )
		{
			if (jiffies > (start_jiffie + EPCS_TIMEOUT) )
				return (-1);
//			schedule_usleep( 50 );
		}
		tmp = ioread32( (void __iomem *)(&epcs->control) );
		iowrite32( tmp & ~NIOS_SPI_SSO, (void __iomem *)(&epcs->control) );
	}
	return (0);
}

//-----------------------------------------------------------------------------
int epcs_tx (unsigned char c)
//-----------------------------------------------------------------------------
{
	unsigned long start_jiffie;

	start_jiffie = jiffies;
	while ( (ioread32( (void __iomem *)(&epcs->status) ) & NIOS_SPI_TRDY) == 0)
	{
		if (jiffies > (start_jiffie + EPCS_TIMEOUT) )
			return (-1);
//		schedule_usleep( 50 );
	}
	iowrite32( c, (void __iomem *)(&epcs->txdata) );
	return (0);
}

//-----------------------------------------------------------------------------
int epcs_rx (void)
//-----------------------------------------------------------------------------
{
	unsigned long start_jiffie;

	start_jiffie = jiffies;
	while ((ioread32( (void __iomem *)(&epcs->status) ) & NIOS_SPI_RRDY) == 0)
	{
		if (jiffies > (start_jiffie + EPCS_TIMEOUT) )
			return (-1);
//		schedule_usleep( 50 );
	}
	return (ioread32( (void __iomem *)(&epcs->rxdata) ));
}

//-----------------------------------------------------------------------------
static unsigned char bitrev[] = {
//-----------------------------------------------------------------------------
	0x00, 0x08, 0x04, 0x0c, 0x02, 0x0a, 0x06, 0x0e,
	0x01, 0x09, 0x05, 0x0d, 0x03, 0x0b, 0x07, 0x0f
};

//-----------------------------------------------------------------------------
static unsigned char epcs_bitrev (unsigned char c)
//-----------------------------------------------------------------------------
{
	unsigned char val;

	val  = bitrev[c>>4];
	val |= bitrev[c & 0x0f]<<4;
	return (val);
}

//-----------------------------------------------------------------------------
static void epcs_rcv (unsigned char *dst, int len)
//-----------------------------------------------------------------------------
{
	while (len--) {
		epcs_tx (0);
		*dst++ = (unsigned char)epcs_rx ();
	}
}

//-----------------------------------------------------------------------------
static void epcs_rrcv (unsigned char *dst, int len)
//-----------------------------------------------------------------------------
{
	while (len--) {
		epcs_tx (0);
		*dst++ = epcs_bitrev ((unsigned char)epcs_rx ());
	}
}

//-----------------------------------------------------------------------------
static void epcs_snd (unsigned char *src, int len)
//-----------------------------------------------------------------------------
{
	while (len--) {
		epcs_tx (*src++);
		epcs_rx ();
	}
}

//-----------------------------------------------------------------------------
static void epcs_rsnd (unsigned char *src, int len)
//-----------------------------------------------------------------------------
{
	while (len--) {
		epcs_tx (epcs_bitrev (*src++));
		epcs_rx ();
	}
}

//-----------------------------------------------------------------------------
static void epcs_rsnd_ex (unsigned char *src, int len)
//-----------------------------------------------------------------------------
{
	while (len--)
		epcs_tx (epcs_bitrev (*src++));
}

//-----------------------------------------------------------------------------
static void epcs_wr_enable (void)
//-----------------------------------------------------------------------------
{
	epcs_cs (1);
	epcs_tx (EPCS_WRITE_ENA);
	epcs_rx ();
	epcs_cs (0);
}

//-----------------------------------------------------------------------------
static unsigned char epcs_status_rd (void)
//-----------------------------------------------------------------------------
{
	unsigned char status;

	epcs_cs (1);
	epcs_tx (EPCS_READ_STAT);
	epcs_rx ();
	epcs_tx (0);
	status = (unsigned char)epcs_rx ();
	epcs_cs (0);
	return (status);
}

//-----------------------------------------------------------------------------
static void epcs_status_wr (unsigned char status)
//-----------------------------------------------------------------------------
{
	epcs_wr_enable ();
	epcs_cs (1);
	epcs_tx (EPCS_WRITE_STAT);
	epcs_rx ();
	epcs_tx (status);
	epcs_rx ();
	epcs_cs (0);
	return;
}

//-----------------------------------------------------------------------------
// Device information
//-----------------------------------------------------------------------------

//static struct epcs_devinfo_t devinfo[] = {
//	{ "EPCS1 ", 0x10, 17, 4, 15, 8, 0x0c },
//	{ "EPCS4 ", 0x12, 19, 8, 16, 8, 0x1c },
//	{ 0, 0, 0, 0, 0, 0 }
//};
//typedef struct epcs_devinfo_t {
//	const char 	*name;		/* Device name */
//	unsigned char	id;		/* Device silicon id */
//	unsigned char	size;		/* Total size log2(bytes)*/
//	unsigned char	num_sects;	/* Number of sectors */
//	unsigned char	sz_sect;	/* Sector size log2(bytes) */
//	unsigned char	sz_page;	/* Page size log2(bytes) */
//	unsigned char   prot_mask;	/* Protection mask */
//}epcs_devinfo_t;

static struct epcs_devinfo_t devinfo[] = {
	{ "M25p40", 0x12 , 22, 8, 19, 8, 0x1c },
	{ "M25p16 ", 0x14 , 24, 32, 19, 8, 0x1c },
	{ 0, 0, 0, 0, 0, 0, 0 }

};

//-----------------------------------------------------------------------------
int epcs_reset (void)
//-----------------------------------------------------------------------------
{
	/* When booting from an epcs controller, the epcs bootrom
	 * code may leave the slave select in an asserted state.
	 * This causes two problems: (1) The initial epcs access
	 * will fail -- not a big deal, and (2) a software reset
	 * will cause the bootrom code to hang since it does not
	 * ensure the select is negated prior to first access -- a
	 * big deal. Here we just negate chip select and everything
	 * gets better :-)
	 */
	epcs_cs (0); /* Negate chip select */
	return (0);
}

//-----------------------------------------------------------------------------
epcs_devinfo_t *epcs_dev_find (void)
//-----------------------------------------------------------------------------
{
	unsigned char buf[4];
	unsigned char id;
	int i;
	struct epcs_devinfo_t *dev = NULL;

	/* Read silicon id requires 3 "dummy bytes" before it's put
	 * on the wire.
	 */
	buf[0] = EPCS_READ_ID;
	buf[1] = 0;
	buf[2] = 0;
	buf[3] = 0;

	epcs_cs (1);
	epcs_snd (buf,4);
	epcs_rcv (buf,1);
	if (epcs_cs (0) == -1)
		return (NULL);
	id = buf[0];

	/* Find the info struct */
	i = 0;
	while (devinfo[i].name) {
		if (id == devinfo[i].id) {
			dev = &devinfo[i];
			break;
		}
		i++;
	}

	return (dev);
}

//-----------------------------------------------------------------------------
// Misc Utilities
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
int epcs_cfgsz (void)
//-----------------------------------------------------------------------------
{
	int sz = 0;
	unsigned char buf[128];
	unsigned char *p;
	struct epcs_devinfo_t *dev = epcs_dev_find ();

	if (!dev)
		return (-1);

	/* Read in the first 128 bytes of the device */
	buf[0] = EPCS_READ_BYTES;
	buf[1] = 0;
	buf[2] = 0;
	buf[3] = 0;

	epcs_cs (1);
	epcs_snd (buf,4);
	epcs_rrcv (buf, sizeof(buf));
	epcs_cs (0);

	/* Search for the starting 0x6a which is followed by the
	 * 4-byte 'register' and 4-byte bit-count.
	 */
	p = buf;
	while (p < buf + sizeof(buf)-8) {
		if ( *p == 0x6a ) {
			/* Point to bit count and extract */
			p += 5;
			sz = *p++;
			sz |= *p++ << 8;
			sz |= *p++ << 16;
			sz |= *p++ << 24;
			/* Convert to byte count */
			sz += 7;
			sz >>= 3;
		} else if (*p == 0xff) {
			/* 0xff is ok ... just skip */
			p++;
			continue;
		} else {
			/* Not 0xff or 0x6a ... something's not
			 * right ... report 'unknown' (sz=0).
			 */
			break;
		}
	}
	return (sz);
}

//-----------------------------------------------------------------------------
int epcs_erase (unsigned start, unsigned end)
//-----------------------------------------------------------------------------
{
	unsigned off, sectsz;
	unsigned char buf[4];
	struct epcs_devinfo_t *dev = epcs_dev_find ();

	if (!dev || (start>end))
		return (-1);

	/* Erase the requested sectors. An address is required
	 * that lies within the requested sector -- we'll just
	 * use the first address in the sector.
	 */
	printk(" epcs erasing sector %d to %d", start, end+1);
	sectsz = (1 << dev->sz_sect);
	while (start <= end) {
		off = start * sectsz;
		start++;

		buf[0] = EPCS_ERASE_SECT;
		buf[1] = (unsigned char)(off >> 16);
		buf[2] = (unsigned char)(off >> 8);
		buf[3] = (unsigned char)off;

		epcs_wr_enable ();
		epcs_cs (1);
		epcs_snd (buf,4);
		epcs_cs (0);

		printk(" ."); /* Some user feedback */

		/* Wait for erase to complete */
		while (epcs_status_rd() & EPCS_STATUS_WIP)
			;
	}
	printk("  done.\n");
	return (0);
}

//-----------------------------------------------------------------------------
int epcs_read (unsigned long addr, unsigned long off, unsigned long cnt)
//-----------------------------------------------------------------------------
{
	unsigned char buf[4];
	struct epcs_devinfo_t *dev = epcs_dev_find ();

	if (!dev)
		return (-1);

	buf[0] = EPCS_READ_BYTES;
	buf[1] = (unsigned char)(off >> 16);
	buf[2] = (unsigned char)(off >> 8);
	buf[3] = (unsigned char)off;

	epcs_cs (1);
	epcs_snd (buf,4);
	epcs_rrcv ((unsigned char *)addr, cnt);
	epcs_cs (0);

	return (0);
}

//-----------------------------------------------------------------------------
int epcs_write (unsigned long addr, unsigned long off, unsigned long cnt)
//-----------------------------------------------------------------------------
{
	unsigned long wrcnt;
	unsigned pgsz;
	unsigned char buf[4];
	struct epcs_devinfo_t *dev = epcs_dev_find ();

	if (!dev)
		return (-1);

	pgsz = (1<<dev->sz_page);
	while (cnt) {
		if (off % pgsz)
			wrcnt = pgsz - (off % pgsz);
		else
			wrcnt = pgsz;
		wrcnt = (wrcnt > cnt) ? cnt : wrcnt;

		buf[0] = EPCS_WRITE_BYTES;
		buf[1] = (unsigned char)(off >> 16);
		buf[2] = (unsigned char)(off >> 8);
		buf[3] = (unsigned char)off;

		epcs_wr_enable ();
		epcs_cs (1);
		epcs_snd (buf,4);
		//epcs_rsnd ((unsigned char *)addr, wrcnt);
		epcs_rsnd_ex ((unsigned char *)addr, wrcnt);
		epcs_cs (0);

		/* Wait for write to complete */
		while (epcs_status_rd() & EPCS_STATUS_WIP)
			;

		cnt -= wrcnt;
		off += wrcnt;
		addr += wrcnt;
	}

	return (0);
}

//-----------------------------------------------------------------------------
int epcs_bulk_erase ()
//-----------------------------------------------------------------------------
{
	unsigned long wrcnt;
	unsigned pgsz;
	unsigned char buf[4];
	struct epcs_devinfo_t *dev = epcs_dev_find ();

	if (!dev)
		return (-1);

	buf[0] = EPCS_ERASE_BULK;
	buf[1] = 0;
	buf[2] = 0;
	buf[3] = 0;

	epcs_wr_enable ();
	epcs_cs (1);
	epcs_snd (buf,1);
	epcs_cs (0);

	/* Wait for write to complete */
	while (epcs_status_rd() & EPCS_STATUS_WIP)
		;

	return (0);
}

//-----------------------------------------------------------------------------
int epcs_verify (unsigned long addr, unsigned long off, unsigned long cnt, unsigned long *err)
//-----------------------------------------------------------------------------
{
	unsigned long i, rdcnt;
	unsigned char buf[256];
	unsigned char *start,*end;

	start = end = (unsigned char *)addr;
	while (cnt) {
		rdcnt = (cnt>sizeof(buf)) ? sizeof(buf) : cnt;
		epcs_read ((unsigned long)buf, off, rdcnt);
		for (i=0; i<rdcnt; i++) {
			if (*end != buf[i]) {
				*err = end - start;
				return(-1);
			}
			end++;
		}
		cnt -= rdcnt;
		off += rdcnt;
	}
	return (0);
}

//-----------------------------------------------------------------------------
static int epcs_sect_erased (int sect, unsigned *offset, struct epcs_devinfo_t *dev)
//-----------------------------------------------------------------------------
{
	unsigned char buf[128];
	unsigned off, end;
	unsigned sectsz;
	int i;

	sectsz = (1 << dev->sz_sect);
	off = sectsz * sect;
	end = off + sectsz;

	while (off < end) {
		epcs_read ((unsigned long)buf, off, sizeof(buf));
		for (i=0; i < sizeof(buf); i++) {
			if (buf[i] != 0xff) {
				*offset = off + i;
				return (0);
			}
		}
		off += sizeof(buf);
	}
	return (1);
}



//-----------------------------------------------------------------------------
// Commands
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
static void do_epcs_info(struct epcs_devinfo_t *dev, int argc, char *argv[])
//-----------------------------------------------------------------------------
{
	int i;
	unsigned char stat;
	unsigned tmp;
	int erased;

	/* Basic device info */
	printk(" %s: %d kbit (%d sectors x %d kbytes," " %d bytes/page)\n", dev->name, 1 << (dev->size-10),	dev->num_sects, 1 << (dev->sz_sect-10), 1 << dev->sz_page );
	/* Status -- for now protection is all-or-nothing */
	stat = epcs_status_rd();
	printk(" status: 0x%02x (WIP:%d, WEL:%d, PROT:%s)\n", stat, (stat & EPCS_STATUS_WIP) ? 1 : 0, (stat & EPCS_STATUS_WEL) ? 1 : 0, (stat & dev->prot_mask) ? "on" : "off" );

	/* Configuration  */
	tmp = epcs_cfgsz ();
	if (tmp) {
		printk(" config: 0x%06x (%d) bytes\n", tmp, tmp );
	} else {
		printk(" config: unknown\n" );
	}

	/* Sector info */
	for (i=0; i<dev->num_sects; i++) {
		erased = epcs_sect_erased (i, &tmp, dev);
		printk("      %d: %06x ",
			i, i*(1<<dev->sz_sect) );
		if (erased)
			printk(" erased\n");
		else
			printk(" data @ 0x%06x\n", tmp);
	}

	return;
}

//-----------------------------------------------------------------------------
static void do_epcs_erase (struct epcs_devinfo_t *dev, unsigned long start, unsigned long end)
//-----------------------------------------------------------------------------
{
	if ((start >= dev->num_sects) || (start > end)) {
		printk(" epcs: invalid sector range: [%d:%d]\n",
			start, end );
		return;
	}

	epcs_erase (start, end);

	return;
}

//-----------------------------------------------------------------------------
static void do_epcs_protect( struct epcs_devinfo_t *dev, int argc, char *argv[])
//-----------------------------------------------------------------------------
{
	unsigned char stat;

	/* For now protection is all-or-nothing to keep things
	 * simple. The protection bits don't map in a linear
	 * fashion ... and we would rather protect the bottom
	 * of the device since it contains the config data and
	 * leave the top unprotected for app use. But unfortunately
	 * protection works from top-to-bottom so it does
	 * really help very much from a software app point-of-view.
	 */
	if (argc < 3) {
		printk(" USAGE: epcs protect on | off\n");
		return;
	}
	if (!dev)
		return;

	/* Protection on/off is just a matter of setting/clearing
	 * all protection bits in the status register.
	 */
	stat = epcs_status_rd ();
	if (strcmp ("on", argv[2]) == 0) {
		stat |= dev->prot_mask;
	} else if (strcmp ("off", argv[2]) == 0 ) {
		stat &= ~dev->prot_mask;
	} else {
		printk(" epcs: unknown protection: %s\n", argv[2]);
		return;
	}
	epcs_status_wr (stat);
	/* Wait for write to complete */
	while (epcs_status_rd() & EPCS_STATUS_WIP)
		;
	return;
}

//-----------------------------------------------------------------------------
static void do_epcs_read (struct epcs_devinfo_t *dev, unsigned long addr, unsigned long off, unsigned long cnt)
//-----------------------------------------------------------------------------
{
	unsigned long sz;

/*	if (argc < 5) {
		printk(" USAGE: epcs read addr offset count\n");
		return;
	}*/

	sz = 1 << dev->size;
	if (off > sz) {
		printk(" offset is greater than device size" "... aborting.\n");
		return;
	}
	if ((off + cnt) > sz) {
		printk(" request exceeds device size" "... truncating.\n");
		cnt = sz - off;
	}
	printk(" epcs: read %08lx <- %06lx (0x%lx bytes)\n", addr, off, cnt);
	epcs_read (addr, off, cnt);

	return;
}

//-----------------------------------------------------------------------------
static void do_epcs_write (struct epcs_devinfo_t *dev, unsigned long addr, unsigned long off, unsigned long cnt)
//-----------------------------------------------------------------------------
{
	unsigned long sz;
	unsigned long err;

	/*if (argc < 5) {
		printk(" USAGE: epcs write addr offset count\n");
		return;
	}*/
	if ((epcs_status_rd() & dev->prot_mask) != 0) {
		printk("epcs: device protected.\n");
		return;
	}

	sz = 1 << dev->size;
	if (off > sz) {
		printk(" offset is greater than device size"
			"... aborting.\n");
		return;
	}
	if ((off + cnt) > sz) {
		printk(" request exceeds device size"
			"... truncating.\n");
		cnt = sz - off;
	}
	printk(" epcs: write %08lx -> %06lx (0x%lx bytes)\n",
			addr, off, cnt);
	epcs_write (addr, off, cnt);
	if (epcs_verify (addr, off, cnt, &err) != 0)
	{
		printk(" epcs: write error at offset %06lx\n", err);
	}
	else
	{
		printk(" epcs: write %08lx -> %06lx (0x%lx bytes) verify ok\n",
					addr, off, cnt);
	}
	return;
}

//-----------------------------------------------------------------------------
static void do_epcs_verify (struct epcs_devinfo_t *dev, unsigned long addr, unsigned long off, unsigned long cnt)
//-----------------------------------------------------------------------------
{
	unsigned long sz;
	unsigned long err;

	/*if (argc < 5) {
		printk(" USAGE: epcs verify addr offset count\n");
		return;
	}*/

	sz = 1 << dev->size;
	if (off > sz) {
		printk(" offset is greater than device size" "... aborting.\n");
		return;
	}
	if ((off + cnt) > sz) {
		printk(" request exceeds device size" "... truncating.\n");
		cnt = sz - off;
	}
	printk(" epcs: verify %08lx -> %06lx (0x%lx bytes)\n",	addr, off, cnt);
	if (epcs_verify (addr, off, cnt, &err) != 0)
		printk(" epcs: verify error at offset %06lx\n", err);

	return;
}

//-----------------------------------------------------------------------------
void set_epcs_base ( void* base )
//-----------------------------------------------------------------------------
{
	epcs = (nios_spi_t *)base;
}

//-----------------------------------------------------------------------------
void write_data_to_flash ( void* base, void* flashdata, int bytes )
//-----------------------------------------------------------------------------
{
	struct epcs_devinfo_t *dev;
	printk(" +%s base %p\n", __FUNCTION__, base );
	epcs = (nios_spi_t *)base;
	dev = epcs_dev_find ();
	if (!dev )
		return ;
	do_epcs_info( dev, 0, 0 );
#if DO_NOT_UPDATE_FLASH
	epcs_bulk_erase( );
	do_epcs_write( dev, (unsigned long)flashdata, 0, bytes );
#endif
}
